from __future__ import print_function, division
import numpy as np
from scipy.linalg import lstsq
from scipy.optimize import curve_fit
from sklearn import linear_model, datasets


def berv_telluric_curvefit(xdata, p0, p1, p2):
    return xdata[0] * p0 + xdata[1] * p1 + p2


def berv_linear_curve_fit(airmass, berv, logi_array, sigi_array, n_axis):
    C = []
    pams = np.zeros(3)-0.1
    ltel = np.empty(n_axis)
    shift = np.empty(n_axis)
    zero = np.empty(n_axis)
    airmass_zero = 0. #np.average(airmass)
    berv_zero = 0. #np.average(berv)

    for ii in xrange(0, n_axis):
        popt, pcov = curve_fit(berv_telluric_curvefit,
                               [airmass-airmass_zero, berv-berv_zero],
                               logi_array[:, ii],
                               p0=pams,
                               sigma = sigi_array[:, ii],
                               bounds=([-np.inf, -np.inf, -np.inf], [0.000, np.inf, np.inf]))

        ltel[ii] = popt[0]
        shift[ii] = popt[1]
        zero[ii] = popt[2]

    return ltel, shift, zero


def berv_linear_lstsq(airmass, berv, logi_array):
    A = np.c_[airmass, berv, np.ones(logi_array.shape[0])]
    C, _, _, _ = lstsq(A, logi_array)    # coefficients
    return C[0], C[1], C[2]


def airmass_telluric_curvefit(xdata, p0, p1):
    return xdata * p0 + p1





def airmass_linear_curve_fit_ransac(airmass, logi_array, sigi_array, n_axis):

    pams = np.zeros(2)
    ltel = np.empty(n_axis)
    zero = np.empty(n_axis)
    airmass_zero = np.average(airmass)

    airmass_reshape = (airmass-airmass_zero).reshape(-1,1)

    ransac = linear_model.RANSACRegressor()


    for ii in range(0, n_axis):
        y_zero = np.average(logi_array[:, ii])
        ransac.fit(airmass_reshape, logi_array[:, ii]-y_zero)
        ltel[ii] = ransac.estimator_.coef_[0]
        zero[ii] = ransac.estimator_.intercept_ + y_zero

    return ltel, zero


def airmass_linear_curve_fit(airmass, logi_array, sigi_array, n_axis):
    C = []
    pams = np.zeros(2)
    ltel = np.empty(n_axis)
    zero = np.empty(n_axis)
    airmass_zero = np.average(airmass)

    for ii in range(0, n_axis):
        y_zero = np.average(logi_array[:, ii])
        popt, pcov = curve_fit(airmass_telluric_curvefit,
                               airmass-airmass_zero,
                               logi_array[:, ii]-y_zero,
                               p0=pams,
                               sigma = sigi_array[:, ii],
                               bounds=([-np.inf, -np.inf], [np.inf, np.inf]))

        ltel[ii] = popt[0]
        zero[ii] = popt[1]

    return ltel, zero


def berv_linear_curve_fit_modified(airmass, berv, logi_array, sigi_array, n_axis):
    C = []
    pams = np.zeros(3)
    ltel = np.empty(n_axis)
    shift = np.empty(n_axis)
    zero = np.empty(n_axis)
    airmass_zero = np.average(airmass)
    berv_zero = np.average(berv)

    for ii in range(0, n_axis):
        popt, pcov = curve_fit(berv_telluric_curvefit,
                               [airmass-airmass_zero, berv-berv_zero],
                               logi_array[:, ii],
                               p0=pams,
                               sigma = sigi_array[:, ii],
                               bounds=([-np.inf, -np.inf, -np.inf], [np.inf, np.inf, np.inf]))

        ltel[ii] = popt[0]
        shift[ii] = popt[1]
        zero[ii] = popt[2]

    return ltel, shift, zero



def airmass_linear_lstsq(airmass, logi_array):
    A = np.c_[airmass, np.ones(logi_array.shape[0])]
    C, _, _, _ = lstsq(A, logi_array)    # coefficients
    return C[0], C[1]






from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.spectral_subroutines import *
from SLOPpy.subroutines.fit_subroutines import *
from SLOPpy.subroutines.io_subroutines import *
from SLOPpy.subroutines.plot_subroutines import *

__all__ = ["compute_sky_correction", "plot_sky_correction"]

subroutine_name = 'sky_correction'

def compute_sky_correction(config_in):

    night_dict = from_config_get_nights(config_in)

    for night in night_dict:

        try:
            processed = load_from_cpickle('skycorrected_fibA', config_in['output'], night)
            print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Retrieved'))
            continue
        except:

            """ Retrieving the list of observations"""
            lists = load_from_cpickle('lists', config_in['output'], night)

            """ Retrieving the observations and calibration data for fiber B, if they exist"""
            try:
                input_data_B = load_from_cpickle('input_dataset_fibB', config_in['output'], night)
                calib_data_B = load_from_cpickle('calibration_fibB', config_in['output'], night)
                print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Computing'))
            except:
                print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Skipped'))
                continue

        """ Retrieving the observations and calibration data for fiber A"""
        print()
        print("  Retrieving the data for night ", night)
        input_data_A = load_from_cpickle('input_dataset_fibA', config_in['output'], night)
        calib_data_A = load_from_cpickle('calibration_fibA', config_in['output'], night)

        map_orders_A = calib_data_A['fibAB_orders_match']
        map_orders_B = calib_data_B['fibAB_orders_match']
        """map_orders_A = [0,1,2,3] = map_orders_B"""

        processed = {
            'subroutine': 'sky_correction'
        }

        for obs in lists['observations']:
            processed[obs] = {}

        " computing the ratio between the lamp flux of fiber A and  B"
        print("  Computing the ratio between the lamp flux of fiber A and  B")

        processed['ratioAB'] = calib_data_A['lamp'][map_orders_A, :]/calib_data_B['lamp'][map_orders_B, :]

        first_obs = lists['observations'][0]

        wave_difference = \
            input_data_A[first_obs]['wave'][map_orders_A, :] - input_data_B[first_obs]['wave'][map_orders_B, :]

        print("   Wavelength difference between fiber A and B: ", \
            np.average(wave_difference), " +- ", np.std(wave_difference), " \AA")

        if np.abs(np.average(wave_difference)) > 0.006 or np.std(wave_difference) > 0.006:
            raise ValueError("TO BE IMPLEMENTED!!!!!!!   ")
            quit()

        else:
            """ We assume that the relative RV shift between finber A and fiber B in the pixel scale is
            is minimal """

            for obs in lists['observations']:
                print(obs)

                processed[obs]['sky_fibA'] = np.zeros([input_data_A['n_orders'], input_data_A['n_pixels']])
                processed[obs]['sky_fibA'][map_orders_A, :] = \
                    processed['ratioAB'] * input_data_B[obs]['e2ds'][map_orders_B, :]
                processed[obs]['e2ds'] = input_data_A[obs]['e2ds'] - processed[obs]['sky_fibA']
                processed[obs]['e2ds_err'] = np.sqrt(
                    input_data_A[obs]['e2ds_err'][map_orders_A, :] ** 2 +
                    (processed['ratioAB'] * input_data_B[obs]['e2ds_err'][map_orders_B, :]) ** 2)

                """ Zero or negative values are identified, flagged and substituted with another value """
                #replacement = 0.1
                #processed[obs]['null'] = (processed[obs]['e2ds'] <= replacement)
                #processed[obs]['e2ds'][processed[obs]['null']] = replacement

        save_to_cpickle('skycorrected_fibA', processed, config_in['output'], night)
        print()

def plot_sky_correction(config_in, night_input=''):

    night_dict = from_config_get_nights(config_in)

    if night_input=='':
        night_list = night_dict
    else:
        night_list = np.atleast_1d(night_input)

    for night in night_list:

        print("plot_sky_correction                        Night: ", night)

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)

        """ Retrieving the observations and calibration data for fiber B, if they exist"""
        try:
            input_data_B = load_from_cpickle('input_dataset_fibB', config_in['output'], night)
            calib_data_B = load_from_cpickle('calibration_fibB', config_in['output'], night)
        except:
            print("No fiber_B dataset available, skipping sky correction plot")
            continue

        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        input_data_A = load_from_cpickle('input_dataset_fibA', config_in['output'], night)
        processed = load_from_cpickle('skycorrected_fibA', config_in['output'], night)

        colors_properties, colors_plot, colors_scatter = make_color_array_matplotlib3(lists, observational_pams)
        fig, gs, cbax1, ax1, ax2, ax3 = grid_3plot_small()

        for i, obs in enumerate(lists['observations']):
            for k in range(0, input_data_B[obs]['n_orders']):
                if i == 0 and k == 0:
                    ax2.scatter(input_data_B[obs]['wave'][k, :], input_data_B[obs]['e2ds'][k, :],
                                c=colors_scatter['mBJD'][obs], s=2, alpha=0.5, label='Sky observations (ORF)')
                else:
                    ax2.scatter(input_data_B[obs]['wave'][k, :], input_data_B[obs]['e2ds'][k, :],
                                c=colors_scatter['mBJD'][obs], s=2, alpha=0.5)

            for k in range(0, input_data_A[obs]['n_orders']):

                if i == 0 and k == 0:
                    ax1.scatter(input_data_A[obs]['wave'][k, :], input_data_A[obs]['e2ds'][k, :],
                                c=colors_scatter['mBJD'][obs], s=1, alpha=0.5, label='Target observations (ORF)')

                else:
                    ax1.scatter(input_data_A[obs]['wave'][k, :], input_data_A[obs]['e2ds'][k, :],
                                c=colors_scatter['mBJD'][obs], s=1, alpha=0.5)

                ax3.scatter(input_data_A[obs]['wave'][k, :], processed[obs]['e2ds'][k, :],
                            c=colors_scatter['mBJD'][obs], s=1, alpha=0.5)

        ax1.set_title('Night: {0:s} \n Input spectra'.format(night))
        ax1.legend(loc=1)

        ax2.set_title('Sky spectrum from fiber B')

        ax3.set_title('After Sky correction')
        ax3.set_xlabel('$\lambda$ [$\AA$]')

        sm = plt.cm.ScalarMappable(cmap=colors_properties['cmap'], norm=colors_properties['norm']['mBJD'])
        sm.set_array([])  # You have to set a dummy-array for this to work...
        cbar = plt.colorbar(sm, cax=cbax1)
        cbar.set_label('BJD - 2450000.0')
        plt.show()



from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.spectral_subroutines import *
from SLOPpy.subroutines.io_subroutines import *
from SLOPpy.subroutines.fit_subroutines import *
from SLOPpy.subroutines.shortcuts import *
from SLOPpy.subroutines.plot_subroutines import *

from scipy.interpolate import UnivariateSpline

__all__ = ['compute_quick_transmission']

def compute_quick_transmission(config_in, lines_label):

    subroutine_name = 'quick_transmission'

    night_dict = from_config_get_nights(config_in)
    spectral_lines = from_config_get_spectral_lines(config_in)
    lines_dict = spectral_lines[lines_label]
    shared_data = load_from_cpickle('shared', config_in['output'])

    shared_selection = (shared_data['coadd']['wave'] >= lines_dict['range'][0]) \
        & (shared_data['coadd']['wave'] < lines_dict['range'][1])
    binned_selection = (shared_data['binned']['wave'] >= lines_dict['range'][0]) \
        & (shared_data['binned']['wave'] < lines_dict['range'][1])

    transmission_shared= {
        'subroutine': subroutine_name,
        'binned_wave': shared_data['binned']['wave'][binned_selection],
        'binned_step': shared_data['binned']['step'][binned_selection],
        'binned_size': int(np.sum(binned_selection))
    }

    import matplotlib.pyplot as plt

    for night in night_dict:

        #try:
        #    preparation = load_from_cpickle('transmission_preparation',
        #                                  config_in['output'],
        #                                  night)
        #    print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Retrieved'))
        #    continue
        #except:
        #    print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Computing'))
        #    print()

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)

        calib_data = load_from_cpickle('calibration_fibA', config_in['output'], night)
        input_data = retrieve_observations(config_in['output'], night, lists['observations'])
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        if config_in['master-out'].get('use_composite', False):
            master_out = load_from_cpickle('master_out_composite', config_in['output'], night)
            print('  Using composite master-out from all nights')
        else:
            master_out = load_from_cpickle('master_out', config_in['output'], night)

        if config_in['master-out'].get('use_smoothed', False):
            master_out['rescaled'] = master_out['smoothed']
            master_out['rescaled_err'] = master_out['smoothed_err']
            print('  Using smoothed master-out')

        quick_transmission = {
            'subroutine': subroutine_name,
        }

        first_obs = lists['observations'][0]

        quick_transmission['n_pixels'] = input_data[first_obs]['n_pixels']
        quick_transmission['n_orders'] = input_data[first_obs]['n_orders']
        quick_transmission['wave'] = input_data[first_obs]['wave']
        quick_transmission['step'] = input_data[first_obs]['step']
        master_raw = np.zeros([quick_transmission['n_orders'], quick_transmission['n_pixels']])


        for obs in lists['transit_out']:
            master_raw += input_data[obs]['e2ds']
        quick_transmission['master_raw'] = master_raw.copy()
        quick_transmission['master'] = master_raw / np.nanmedian(master_raw)

        blaze = np.ones([quick_transmission['n_orders'], quick_transmission['n_pixels']])


        for obs in lists['observations']:

            quick_transmission[obs] = {}

            transmission_raw = input_data[obs]['e2ds'] / quick_transmission['master']
            quick_transmission[obs]['transmission_raw'] = transmission_raw.copy()
            quick_transmission[obs]['transmission'] = transmission_raw / np.nanmedian(transmission_raw)
            quick_transmission[obs]['wave'] = input_data[obs]['wave'] #Added for plotting purpose only


            quick_transmission[obs]['binned'] = \
                rebin_2d_to_1d(quick_transmission['wave'],
                            quick_transmission['step'],
                            quick_transmission[obs]['transmission'],
                            blaze,
                            transmission_shared['binned_wave'],
                            transmission_shared['binned_step'],
                            rv_shift=0,
                            preserve_flux=False)

            #plt.scatter(quick_transmission['wave'],
            #            quick_transmission[obs]['transmission'],
            #            c='C1', s=1, zorder=3, alpha=0.25)

        for obs in lists['transit_out']:

            plt.scatter(transmission_shared['binned_wave'],
                        quick_transmission[obs]['binned'],
                        c='b', s=1, zorder=10, alpha=0.5)

        for obs in lists['transit_full']:

            plt.scatter(transmission_shared['binned_wave'],
                        quick_transmission[obs]['binned']-0.04,
                        c='r', s=1, zorder=10, alpha=0.5)

        plt.xlim(transmission_shared['binned_wave'][0], transmission_shared['binned_wave'][-1])
        plt.show()


    print()
    """ Keep going from here after preparation, unless the subroutines has been called just
        to preform the data preparation step
    """




from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.spectral_subroutines import *
from SLOPpy.subroutines.fit_subroutines import *
from SLOPpy.subroutines.io_subroutines import *
from SLOPpy.subroutines.shortcuts import *
from SLOPpy.subroutines.plot_subroutines import *

__all__ = ["compute_pca_preparation"]

def compute_pca_preparation(config_in, append_name=None):

    if append_name:
        subroutine_name = 'pca_preparation_' + append_name
        filename = 'pca_preparation_' + append_name
    else:
        subroutine_name = 'pca_preparation'
        filename = 'pca_preparation'

    night_dict = from_config_get_nights(config_in)

    preparation_dict = {
        'fit_iters': 5,
        'fit_order': 3,
        'fit_sigma': 3
    }


    for night in night_dict:

        try:
            preparation = load_from_cpickle(filename, config_in['output'], night)
            print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Retrieved'))
            continue
        except:
            print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Computing'))
            print()

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)

        """ Retrieving input and calibration data """
        calib_data = load_from_cpickle('calibration_fibA', config_in['output'], night)
        input_data = retrieve_observations(config_in['output'], night, lists['observations'],
                                           use_refraction=True, use_telluric=False, use_interstellar=False,
                          use_telluric_spline= False)
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)


        obs_ref =lists['observations'][0]
        n_obs = len(lists['observations'])

        n_orders = input_data[obs_ref]['n_orders']
        n_pixels = input_data[obs_ref]['n_pixels']

        stack_wave = np.zeros([n_obs, n_orders, n_pixels], dtype=np.double)
        stack_e2ds = np.zeros([n_obs, n_orders, n_pixels], dtype=np.double)
        stack_e2ds_err = np.zeros([n_obs, n_orders, n_pixels], dtype=np.double)
        stack_bjd = np.zeros(n_obs, dtype=np.double)
        stack_airmass = np.zeros(n_obs, dtype=np.double)

        for i_obs, obs in enumerate(lists['observations']):

            blaze_wave_refactoring = 1. / calib_data['blaze'] / (input_data[obs]['step']/np.median(input_data[obs]['step']))

            stack_wave[i_obs, :, :] = input_data[obs]['wave']
            stack_e2ds[i_obs, :, :] = input_data[obs]['e2ds'] * blaze_wave_refactoring
            stack_e2ds_err[i_obs, :, :] = input_data[obs]['e2ds_err'] * blaze_wave_refactoring

            stack_bjd[i_obs] = input_data[obs]['BJD']
            stack_airmass[i_obs] = input_data[obs]['AIRMASS']

            median = np.nanmedian(stack_e2ds[i_obs, :, :], axis=1)

            for i_orders in range(0, n_orders):
                stack_e2ds[i_obs, i_orders, :] /= median[i_orders]
                stack_e2ds_err[i_obs, i_orders, :] /= median[i_orders]

        poly_flag = (stack_e2ds > 0.001)
        #poly_flag[:, :, :20]= False
        #poly_flag[:, :, 20:]= False

        stack_polyfit = np.zeros_like(stack_e2ds)

        for i_orders in range(0,n_orders):

            order_wave = stack_wave[:, i_orders, :]
            order_e2ds = stack_e2ds[:, i_orders, :]
            order_flag = poly_flag[:, i_orders, :]

            for n_iter in range(0, preparation_dict['fit_iters']):

                coeff_order = np.polynomial.chebyshev.chebfit(
                    order_wave[order_flag],
                    order_e2ds[order_flag],
                    preparation_dict['fit_order'])

                fit_order = \
                    np.polynomial.chebyshev.chebval(order_wave, coeff_order)
                fit_shaped = np.reshape(fit_order, np.shape(order_wave))
                residuals = order_e2ds - fit_shaped

                if n_iter < preparation_dict['fit_iters'] - 1:
                    std = np.std(residuals[order_flag])
                    order_flag = (order_flag) & (residuals > -preparation_dict['fit_sigma'] * std)

            stack_e2ds[:, i_orders, :]/=fit_shaped
            stack_e2ds_err[:, i_orders, :]/=fit_shaped
            stack_polyfit[:, i_orders, :] =fit_shaped


            #plt.imshow(stack_e2ds[:, i_orders, :], interpolation='none', aspect='auto', vmin=0.25, vmax=1.5)
            #plt.colorbar()
            #plt.show()

        preparation = {
            'stack_wave': stack_wave,
            'stack_e2ds': stack_e2ds,
            'stack_e2ds_err': stack_e2ds_err,
            'stack_bjd': stack_e2ds,
            'stack_airmass': stack_e2ds,
            'stack_polyfit': stack_polyfit,
            'frame': {
                     'n_obs': n_obs,
                     'n_orders': n_orders,
                     'n_pixels': n_pixels,
                },
            'fit_pams': preparation_dict
        }
        save_to_cpickle(filename, preparation, config_in['output'], night)





import numpy as np
import matplotlib.pyplot as plt
import cPickle as pickle

#w,f1,f2,f3,fall,sigma=np.loadtxt('/home/sicilia/Astro/Wyttenbach/HD189733_589289_1200_transmission_spectrum_TransSpec_atmcorr.rdb',unpack=True,skiprows=2)
#w,t1,sigma=np.loadtxt('/home/sicilia/Astro/Wyttenbach/transmission_spectrum_1.rdb',unpack=True)
#w,t2,sigma=np.loadtxt('/home/sicilia/Astro/Wyttenbach/transmission_spectrum_2.rdb',unpack=True)
#w,t3,sigma=np.loadtxt('/home/sicilia/Astro/Wyttenbach/transmission_spectrum_3.rdb',unpack=True)
#w,t,sigma=np.loadtxt('/home/sicilia/Astro/HD189733/list/average_transmission_spectrum2.txt',unpack=True)
#w,t,sigma=np.loadtxt('/home/sicilia/Astro/Wyttenbach/test.rdb',unpack=True)
#n, w, t, sigma = np.loadtxt('/home/sicilia/Astro/Wyttenbach/Wyttenbach_by_Casasayas.txt',unpack=True)
#our_average =  pickle.load(open('/home/sicilia/Astro/HD189733/list/HD1897333_HARPS_transmission_planetRF_average.p', "rb"))
#data_night1 = pickle.load(open('/home/sicilia/Astro/HD189733/list/HD1897333_HARPS_2006-09-07_transmission_planetRF_second_correction.p',"rb"))
#data_night2 = pickle.load(open('/home/sicilia/Astro/HD189733/list/HD1897333_HARPS_2007-07-19_transmission_planetRF_second_correction.p',"rb"))
data_night3 = pickle.load(open('/home/sicilia/Astro/HD189733/list/HD1897333_HARPS_2007-08-28_transmission_planetRF_second_correction.p',"rb"))

'''
w=our_average['wave']
t=(our_average['average']-1.)*100
t[t>1.1]=0.
sigma=our_average['average_err']
'''
w=data_night3['wave']
t=(data_night3['average']-1.)*100
t[t>1.1]=0.
sigma=data_night3['average_err']


#t = (t + 1)
#sigma = sigma/100

#central band
#C6_D2 = lambda wl: ((wl >= 5888.416) and (wl <= 5891.396)) or ((wl >= 5894.389) and (wl <= 5897.369))

w1=5889.906
w2=5895.879
wc=5892.89

#shifting the spectrum and not the passbands
#w = w - 0.04
#w = w + 0.16
#w = w + 0.075
#w = w + 0.05

#blue band & red band
B = (w >= 5874.89) & (w <= 5886.89)
R = (w >= 5898.89) & (w <= 5910.89)

#red band for Casasayas
#R = (w >= 5898.89) & (w <= 5907.89)

#alla c6 aggiungo e sottraggo 1.48
C12 = (w >= 5886.8925) & (w <= 5898.8925)
C6_D2 = (w >= 5888.426) & (w <= 5891.386)
C6_D1 = (w >= 5894.399) & (w <= 5897.359)
C3_D2 = (w >= 5889.156) & (w <= 5890.656)
C3_D1 = (w >= 5895.129) & (w <= 5896.629)
C1_5_D2 = (w >= 5889.531) & (w <= 5890.281)
C1_5_D1 = (w >= 5895.504) & (w <= 5896.254)
C0_75_D2 = (w >= 5889.718) & (w <= 5890.094)
C0_75_D1 = (w >= 5895.691) & (w <= 5896.067)
C0_375_D2 = (w >= 5889.812) & (w <= 5890.000)
C0_375_D1 = (w >= 5895.785) & (w <= 5895.973)

'''
#including -0.16 A
B = (w >= 5874.73) & (w <= 5886.73)
R = (w >= 5898.73) & (w <= 5910.73)

C12 = (w >= 5886.73) & (w <= 5898.73)
C6_D2 = (w >= 5888.246) & (w <= 5891.246)
C6_D1 = (w >= 5894.219) & (w <= 5897.219)
C3_D2 = (w >= 5888.996) & (w <= 5890.496)
C3_D1 = (w >= 5894.969) & (w <= 5896.469)
C1_5_D2 = (w >= 5889.371) & (w <= 5890.121)
C1_5_D1 = (w >= 5895.344) & (w <= 5896.094)
C0_75_D2 = (w >= 5889.558) & (w <= 5889.934)
C0_75_D1 = (w >= 5895.531) & (w <= 5895.907)
C0_375_D2 = (w >= 5889.652) & (w <= 5889.840)
C0_375_D1 = (w >= 5895.625) & (w <= 5895.813)


#sottraendo le bande come dice W e meno 0.16
C12 = (w >= 5886.73) & (w <= 5898.73)
C6_D2 = (w >= 5886.746) & (w <= 5892.746)
C6_D1 = (w >= 5892.719) & (w <= 5898.719)
C3_D2 = (w >= 5888.246) & (w <= 5891.246)
C3_D1 = (w >= 5894.219) & (w <= 5897.219)
C1_5_D2 = (w >= 5888.996) & (w <= 5890.496)
C1_5_D1 = (w >= 5894.969) & (w <= 5896.469)
C0_75_D2 = (w >= 5889.371) & (w <= 5890.121)
C0_75_D1 = (w >= 5895.344) & (w <= 5896.094)
C0_375_D2 = (w >= 5889.558) & (w <= 5889.934)
C0_375_D1 = (w >= 5895.531) & (w <= 5895.907)

'''

flux_C12, sum_weights_C12 = np.average(t[C12],axis=0,weights=1/sigma[C12]**2,returned=True)
flux_C6_D2, sum_weights_C6_D2 = np.average(t[C6_D2],axis=0,weights=1/sigma[C6_D2]**2,returned=True)
#flux_C6_D2, sum_weights_C6_D2 = np.average((np.array(filter(C6_D2, t))),weights=1/np.array(filter(C6_D2, sigma)**2),returned=True)
flux_C6_D1, sum_weights_C6_D1 = np.average(t[C6_D1],axis=0,weights=1/sigma[C6_D1]**2,returned=True)
flux_C3_D2, sum_weights_C3_D2 = np.average(t[C3_D2],axis=0,weights=1/sigma[C3_D2]**2,returned=True)
flux_C3_D1, sum_weights_C3_D1 = np.average(t[C3_D1],axis=0,weights=1/sigma[C3_D1]**2,returned=True)
flux_C1_5_D2, sum_weights_C1_5_D2 = np.average(t[C1_5_D2],axis=0,weights=1/sigma[C1_5_D2]**2,returned=True)
flux_C1_5_D1, sum_weights_C1_5_D1 = np.average(t[C1_5_D1],axis=0,weights=1/sigma[C1_5_D1]**2,returned=True)
flux_C0_75_D2, sum_weights_C0_75_D2 = np.average(t[C0_75_D2],axis=0,weights=1/sigma[C0_75_D2]**2,returned=True)
flux_C0_75_D1, sum_weights_C0_75_D1 = np.average(t[C0_75_D1],axis=0,weights=1/sigma[C0_75_D1]**2,returned=True)
flux_C0_375_D2, sum_weights_C0_375_D2 = np.average(t[C0_375_D2],axis=0,weights=1/sigma[C0_375_D2]**2,returned=True)
flux_C0_375_D1, sum_weights_C0_375_D1 = np.average(t[C0_375_D1],axis=0,weights=1/sigma[C0_375_D1]**2,returned=True)
flux_B, sum_weights_B = np.average(t[B],axis=0,weights=1/sigma[B]**2,returned=True)
flux_R, sum_weights_R = np.average(t[R],axis=0,weights=1/sigma[R]**2,returned=True)

deltaC12 = flux_C12 - (flux_B + flux_R)/2
deltaC6_D2 = flux_C6_D2 - (flux_B + flux_R)/2
deltaC6_D1 = flux_C6_D1 - (flux_B + flux_R)/2
deltaC3_D2 = flux_C3_D2 - (flux_B + flux_R)/2
deltaC3_D1 = flux_C3_D1 - (flux_B + flux_R)/2
deltaC1_5_D2 = flux_C1_5_D2 - (flux_B + flux_R)/2
deltaC1_5_D1 = flux_C1_5_D1 - (flux_B + flux_R)/2
deltaC0_75_D2 = flux_C0_75_D2 - (flux_B + flux_R)/2
deltaC0_75_D1 = flux_C0_75_D1 - (flux_B + flux_R)/2
deltaC0_375_D2 = flux_C0_375_D2 - (flux_B + flux_R)/2
deltaC0_375_D1 = flux_C0_375_D1 - (flux_B + flux_R)/2

delta_medio_6 = (deltaC6_D2 + deltaC6_D1)/2
delta_medio_3 = (deltaC3_D2 + deltaC3_D1)/2
delta_medio_1_5 = (deltaC1_5_D2 + deltaC1_5_D1)/2
delta_medio_0_75 = (deltaC0_75_D2 + deltaC0_75_D1)/2
delta_medio_0_375 = (deltaC0_375_D2 + deltaC0_375_D1)/2

sigma_deltaC12 = np.sqrt(1/sum_weights_C12 + 1/(2*sum_weights_B) + 1/(2*sum_weights_R)) * 100
sigma_deltaC6 = np.sqrt(1/sum_weights_C6_D2 + 1/sum_weights_C6_D1 + 1/sum_weights_B + 1/sum_weights_R)/2 * 100
sigma_deltaC3 = np.sqrt(1/sum_weights_C3_D2 + 1/sum_weights_C3_D1 + 1/sum_weights_B + 1/sum_weights_R)/2 * 100
sigma_deltaC1_5 = np.sqrt(1/sum_weights_C1_5_D2 + 1/sum_weights_C1_5_D1 + 1/sum_weights_B + 1/sum_weights_R)/2 * 100
sigma_deltaC0_75 = np.sqrt(1/sum_weights_C0_75_D2 + 1/sum_weights_C0_75_D1 + 1/sum_weights_B + 1/sum_weights_R)/2 * 100
sigma_deltaC0_375 = np.sqrt(1/sum_weights_C0_375_D2 + 1/sum_weights_C0_375_D1 + 1/sum_weights_B + 1/sum_weights_R)/2 * 100

print 'delta(12) =', deltaC12, ' +- ', sigma_deltaC12
print 'delta(6) = ', delta_medio_6, ' +- ', sigma_deltaC6
print 'delta(3) = ', delta_medio_3, ' +- ', sigma_deltaC3
print 'delta(1.5) = ', delta_medio_1_5, ' +- ', sigma_deltaC1_5
print 'delta(0.75) =', delta_medio_0_75, ' +- ', sigma_deltaC0_75
print 'delta(0.375) =', delta_medio_0_375, ' +- ', sigma_deltaC0_375

fig = plt.figure(figsize=(12, 6))
plt.plot(w,t)
plt.axvline(5892.89,c='k')

plt.axvspan(5886.89,5898.89,facecolor='g',alpha=0.3)
plt.axvspan(5874.89,5886.89,facecolor='b',alpha=0.3)
plt.axvspan(5898.89,5910.89,facecolor='r',alpha=0.3)
plt.axvspan(5888.426,5891.386,facecolor='g',alpha=0.4)
plt.axvspan(5894.399,5897.359,facecolor='g',alpha=0.4)
plt.axvspan(5889.156,5890.656,facecolor='g',alpha=0.5)
plt.axvspan(5895.129,5896.629,facecolor='g',alpha=0.5)
plt.axvspan(5889.531,5890.281,facecolor='g',alpha=0.6)
plt.axvspan(5895.504,5896.254,facecolor='g',alpha=0.6)
plt.axvspan(5889.718,5890.094,facecolor='g',alpha=0.7)
plt.axvspan(5895.691,5896.067,facecolor='g',alpha=0.7)
plt.axvspan(5889.812,5890.000,facecolor='g',alpha=0.8)
plt.axvspan(5895.785,5895.973,facecolor='g',alpha=0.8)

'''

plt.axvspan(5874.89,5886.89,facecolor='b',alpha=0.3)
plt.axvspan(5898.89,5910.89,facecolor='r',alpha=0.3)
plt.axvspan(5888.246,5891.246,facecolor='g',alpha=0.4)
plt.axvspan(5894.219,5897.219,facecolor='g',alpha=0.4)
plt.axvspan(5888.996,5890.496,facecolor='g',alpha=0.5)
plt.axvspan(5894.969,5896.469,facecolor='g',alpha=0.5)
plt.axvspan(5889.371,5890.121,facecolor='g',alpha=0.6)
plt.axvspan(5895.344,5896.094,facecolor='g',alpha=0.6)
plt.axvspan(5886.73,5898.73,facecolor='g',alpha=0.3)
plt.axvspan(5889.558,5889.934,facecolor='g',alpha=0.7)
plt.axvspan(5895.531,5895.907,facecolor='g',alpha=0.7)
plt.axvspan(5889.652,5889.840,facecolor='g',alpha=0.8)
plt.axvspan(5895.625,5895.813,facecolor='g',alpha=0.8)
'''
plt.xlabel('$\lambda$ [$\AA$]')
plt.ylabel('R-1')
plt.show()



from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.spectral_subroutines import *
from SLOPpy.subroutines.fit_subroutines import *
from SLOPpy.subroutines.io_subroutines import *
from SLOPpy.subroutines.shortcuts import *

__all__ = ["compute_differential_refraction", "plot_differential_refraction"]

subroutine_name = 'differential_refraction'


def compute_differential_refraction(config_in):
    night_dict = from_config_get_nights(config_in)

    print()

    for night in night_dict:

        try:
            refraction = load_from_cpickle('refraction', config_in['output'], night)
            print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Retrieved'))
            continue
        except:
            print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Computing'))
            print()

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)

        calib_data = load_from_cpickle('calibration_fibA', config_in['output'], night)
        input_data = retrieve_observations(config_in['output'], night, lists['observations'],
                                           use_refraction=False, use_telluric=False)
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        try:
            processed = load_from_cpickle('refraction_processed_halfway', config_in['output'], night)
            refraction = load_from_cpickle('refraction_halfway', config_in['output'], night)
            print("  Starting from intermediate step ")
        except:

            processed = {
                'subroutine': subroutine_name,
                'coadd': {
                    'wave': input_data['coadd']['wave'],
                    'size': input_data['coadd']['size'],
                    'step': input_data['coadd']['step'],
                    #'flux': np.zeros(input_data['coadd']['size'], dtype=np.double),
                    #'flux_err': np.zeros(input_data['coadd']['size'], dtype=np.double)
                }
            }

            refraction = {
                'subroutine': 'differential_refraction',
                'wave': processed['coadd']['wave']
            }

            total_flux = np.empty([len(lists['observations']), input_data['coadd']['size']], dtype=np.double)
            total_wght = np.zeros([len(lists['observations']), input_data['coadd']['size']], dtype=np.double)
            total_mask = np.ones([len(lists['observations']), input_data['coadd']['size']], dtype=bool)

            print("  Chebyshev polynomial order for differential refraction fit: ",
                  observational_pams['refraction_poly_order'])

            print("  Number of iterations: ",
                  observational_pams['refraction_poly_iters'])
            print()

            """ Rebinning of all the spectra """
            for n_obs, obs in enumerate(lists['observations']):

                print("  Spectral rebinning - Processing: ", obs)
                processed[obs] = {}

                """ Rebinning of the spectra in the SRF, except for a fixed constant in order to minimize
                    the difference between  """
                preserve_flux = input_data[obs].get('absolute_flux', True)

                processed[obs]['flux_rebinned_stellarRF'] = \
                    rebin_2d_to_1d(input_data[obs]['wave'],
                                   input_data[obs]['step'],
                                   input_data[obs]['e2ds'],
                                   calib_data['blaze'],
                                   processed['coadd']['wave'],
                                   processed['coadd']['step'],
                                   preserve_flux=preserve_flux,
                                   rv_shift=observational_pams[obs]['rv_shift_ORF2SRF_mod'])

                processed[obs]['err_flux_rebinned_SRF'] = \
                    rebin_2d_to_1d(input_data[obs]['wave'],
                                   input_data[obs]['step'],
                                   input_data[obs]['e2ds_err'],
                                   calib_data['blaze'],
                                   processed['coadd']['wave'],
                                   processed['coadd']['step'],
                                   preserve_flux=preserve_flux,
                                   rv_shift=observational_pams[obs]['rv_shift_ORF2SRF_mod'],
                                   is_error=True)

                """ Zero or negative values are identified, flagged and substituted with another value """
                processed[obs]['flux_rebinned_stellarRF'], \
                processed[obs]['err_flux_rebinned_SRF'], \
                processed[obs]['flux_rebinned_SRF_null'] = \
                    replace_values_errors_with_interpolation_1d(processed[obs]['flux_rebinned_stellarRF'],
                                                                processed[obs]['err_flux_rebinned_SRF'],
                                                                force_positive=True)

                processed[obs]['rescaling'], processed[obs]['rescaled'], processed[obs]['rescaled_err'] = \
                    perform_rescaling(processed['coadd']['wave'],
                                      processed[obs]['flux_rebinned_stellarRF'],
                                      processed[obs]['err_flux_rebinned_SRF'],
                                      observational_pams['wavelength_rescaling'])

                processed[obs]['rescaled_blazed'] = input_data[obs]['e2ds'] \
                                                    / processed[obs]['rescaling'] \
                                                    / calib_data['blaze']

                if obs in lists['telluric']:
                    total_flux[n_obs, :] = processed[obs]['rescaled']
                    total_mask[n_obs, :] = processed[obs]['flux_rebinned_SRF_null']
                    total_wght[n_obs, :] = 1. / (processed[obs]['rescaled_err'] ** 2)

                    # processed['coadd']['flux'] += processed[obs]['flux_rebinned_stellarRF']
                    # """ SNR (assumed to be the square root of the flux) is added in quadrature """
                    # processed['coadd']['flux_err'] += processed[obs]['err_flux_rebinned_SRF'] ** 2
                    print("      Observation added to reference spectrum")

            #masked_array = np.ma.array(total_flux, mask=total_mask)
            #processed['coadd']['rescaled'], sum_weights = np.ma.average(masked_array,
            #                                                            weights=total_wght,
            #                                                            axis=0,
            #                                                            returned=True)
            # processed['coadd']['rescaled'][sum_weights <= 0.0001] = 1.000
            # sum_weights[sum_weights <= 0.0001] = 0.0001
            # processed['coadd']['rescaled_err'] = 1. / np.sqrt(sum_weights)

            masked_array = np.ma.array(total_flux, mask=total_mask)
            rescaled_mask, sum_weights = np.ma.average(masked_array,
                                                       weights=total_wght,
                                                       axis=0,
                                                       returned=True)

            processed['coadd']['rescaled'] = rescaled_mask.filled(0.00)
            sum_weights[sum_weights <= 0.0] = 1.0
            processed['coadd']['rescaled_err'] = 1. / np.sqrt(sum_weights)

            processed['coadd']['rescaled'], processed['coadd']['rescaled_err'], processed['coadd']['null'] = \
                replace_values_errors_with_interpolation_1d(processed['coadd']['rescaled'],
                                                            processed['coadd']['rescaled_err'],
                                                            force_positive=True)

            save_to_cpickle('refraction_processed_halfway', processed, config_in['output'], night)
            save_to_cpickle('refraction_halfway', refraction, config_in['output'], night)

        """ Now each observation is divided by the reference spectrum, after being redshifted in the observer RF
            The result is then used to model the flux variation
        """

        for obs in lists['observations']:

            print("  Division by reference spectrum and fit of the flux variation: ", obs)

            preserve_flux = input_data[obs].get('absolute_flux', True)

            """ Going back to the observer RF and rebinning the spectrum into the observed orders """
            processed[obs]['master_flux'] = \
                rebin_1d_to_2d(processed['coadd']['wave'],
                               processed['coadd']['step'],
                               processed['coadd']['rescaled'],
                               input_data[obs]['wave'],
                               input_data[obs]['step'],
                               preserve_flux=preserve_flux,
                               rv_shift=-observational_pams[obs]['rv_shift_ORF2SRF_mod'])

            processed[obs]['master_ferr'] = \
                rebin_1d_to_2d(processed['coadd']['wave'],
                               processed['coadd']['step'],
                               processed['coadd']['rescaled_err'],
                               input_data[obs]['wave'],
                               input_data[obs]['step'],
                               preserve_flux=preserve_flux,
                               rv_shift=-observational_pams[obs]['rv_shift_ORF2SRF_mod'],
                               is_error=True)

            """ Zero or negative values are identified, flagged and substituted with another value """
            processed[obs]['master_flux'], processed[obs]['master_ferr'], processed[obs]['master_null'] = \
                replace_values_errors_with_interpolation_2d(processed[obs]['master_flux'],
                                                            processed[obs]['master_ferr'],
                                                            less_than=0.001)

            processed[obs]['ratio'] = processed[obs]['rescaled_blazed'] / processed[obs]['master_flux']
            """
            processed[obs]['ratio'] = input_data[obs]['e2ds']\
                                      /processed[obs]['rescaling']\
                                      / (processed[obs]['master_flux'] * calib_data['blaze'])
            """

            refraction[obs] = {}
            refraction[obs]['polyfit_e2ds'] = np.zeros([input_data[obs]['n_orders'], input_data[obs]['n_pixels']])
            processed[obs]['residuals'] = np.zeros([input_data[obs]['n_orders'], input_data[obs]['n_pixels']])
            refraction[obs]['poly_flag'] = np.zeros([input_data[obs]['n_orders'], input_data[obs]['n_pixels']],
                                                    dtype=bool)

            for order in range(0, input_data[obs]['n_orders']):

                order_coeff_name = 'order_' + repr(order)
                refraction[obs]['poly_flag'][order, :] = (processed[obs]['ratio'][order, :] > 0.1)
                refraction[obs]['poly_flag'][order, :50] = False
                refraction[obs]['poly_flag'][order, -50:] = False

                for n_iter in range(0, observational_pams['refraction_poly_iters']):

                    refraction[obs][order_coeff_name] = np.polynomial.chebyshev.chebfit(
                        input_data[obs]['wave'][order, refraction[obs]['poly_flag'][order, :]],
                        processed[obs]['ratio'][order, refraction[obs]['poly_flag'][order, :]],
                        observational_pams['refraction_poly_order'])

                    refraction[obs]['polyfit_e2ds'][order, :] = \
                        np.polynomial.chebyshev.chebval(input_data[obs]['wave'][order, :],
                                                        refraction[obs][order_coeff_name])

                    processed[obs]['residuals'][order, :] = refraction[obs]['polyfit_e2ds'][order, :]\
                                                            - processed[obs]['ratio'][order, :]

                    if n_iter < observational_pams['refraction_poly_iters'] - 1:
                        std = np.std(processed[obs]['residuals'][order, :])
                        refraction[obs]['poly_flag'][order, :] = (refraction[obs]['poly_flag'][order, :]) \
                                                                 & (np.abs(processed[obs]['residuals'][order, :]) <
                                                                    observational_pams['refraction_poly_sigma'] * std)

            processed[obs]['e2ds_corrected'] = input_data[obs]['e2ds'] / refraction[obs]['polyfit_e2ds']
            processed[obs]['e2ds_corrected_err'] = input_data[obs]['e2ds_err'] / refraction[obs]['polyfit_e2ds']

            preserve_flux = input_data[obs].get('absolute_flux', True)

            processed[obs]['flux_rebinned_stellarRF_corrected'] = \
                rebin_2d_to_1d(input_data[obs]['wave'],
                               input_data[obs]['step'],
                               processed[obs]['e2ds_corrected'],
                               calib_data['blaze'],
                               processed['coadd']['wave'],
                               processed['coadd']['step'],
                               preserve_flux=preserve_flux,
                               rv_shift=observational_pams[obs]['rv_shift_ORF2SRF_mod'])

            processed[obs]['err_flux_rebinned_SRF_corrected'] = \
                rebin_2d_to_1d(input_data[obs]['wave'],
                               input_data[obs]['step'],
                               processed[obs]['e2ds_corrected_err'],
                               calib_data['blaze'],
                               processed['coadd']['wave'],
                               processed['coadd']['step'],
                               preserve_flux=preserve_flux,
                               rv_shift=observational_pams[obs]['rv_shift_ORF2SRF_mod'],
                               is_error=True)

            processed[obs]['flux_rebinned_stellarRF_corrected'], \
            processed[obs]['err_flux_rebinned_SRF_corrected'], _ = \
                replace_values_errors_with_interpolation_1d(processed[obs]['flux_rebinned_stellarRF_corrected'],
                                                            processed[obs]['err_flux_rebinned_SRF_corrected'],
                                                            less_than=0.001)

        save_to_cpickle('refraction_processed', processed, config_in['output'], night)
        save_to_cpickle('refraction', refraction, config_in['output'], night)


def plot_differential_refraction(config_in, night_input=''):
    night_dict = from_config_get_nights(config_in)

    if night_input == '':
        night_list = night_dict
    else:
        night_list = np.atleast_1d(night_input)

    for night in night_list:

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)

        """ Retrieving the observations"""
        input_data = retrieve_observations(config_in['output'], night, lists['observations'],
                                           use_refraction=False, use_telluric=False)
        #observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)


        try:
            """ Retrieving the analysis"""
            processed = load_from_cpickle('refraction_processed', config_in['output'], night)
            refraction = load_from_cpickle('refraction', config_in['output'], night)
        except:
            print("  Failed in retrieving processed data")
            return

        """ Creation of the color array, based on the BJD of the observations
        """
        bjd = []
        am = []

        for obs in lists['observations']:
            bjd.append(input_data[obs]['BJD'] - 2450000.0)
            am.append(input_data[obs]['AIRMASS'])

        color_cmap = plt.cm.viridis
        color_norm = plt.Normalize(vmin=bjd[0], vmax=bjd[-1])
        colors = color_cmap(color_norm(np.asarray(bjd)))

        offset = 0.10

        y_limits = [0.8, 1.2]


        """
        fig = plt.figure(figsize=(12, 6))
        gs = GridSpec(2, 2, width_ratios=[50, 1])

        ax1 = plt.subplot(gs[0, 0])
        ax2 = plt.subplot(gs[1, 0], sharex=ax1, sharey=ax1)
        cbax1 = plt.subplot(gs[:, 1])

        for i, obs in enumerate(lists['observations']):
            shift = i/10.0

            for order in range(0, input_data[obs]['n_orders']):
                ax1.scatter(input_data[obs]['wave'][order, :],
                         processed[obs]['rescaled_blazed'][order, :] - shift,
                         c=line_colors[i], s=1, alpha=0.5)

                ax1.plot(input_data[obs]['wave'][order, :],
                         processed[obs]['master_flux'][order, :], - shift,
                         c='k', lw=1)

                ax2.scatter(input_data[obs]['wave'][order, :],
                         processed[obs]['rescaled_blazed'][order, :]/refraction[obs]['polyfit_e2ds'][order, :] - shift,
                         c=line_colors[i], s=1, alpha=0.5)

                ax2.plot(input_data[obs]['wave'][order, :],
                         processed[obs]['master_flux'][order, :], - shift,
                         c='k', lw=1)


        ax1.set_xlim(processed['coadd']['wave'][0], processed['coadd']['wave'][-1])
        ax1.legend(loc=3)
        ax1.set_title('Night: ' + night)

        ax2.set_xlabel('$\lambda$ [$\AA$]')

        sm = plt.cm.ScalarMappable(cmap=cmap, norm=plt.Normalize(vmin=colors[0], vmax=colors[-1]))
        sm.set_array([])  # You have to set a dummy-array for this to work...
        cbar = plt.colorbar(sm, cax=cbax1)
        cbar.set_label('BJD - 2450000.0')
        fig.subplots_adjust(wspace=0.05, hspace=0.4)
        plt.show()
        """

        fig = plt.figure(figsize=(12, 6))
        gs = GridSpec(2, 2, width_ratios=[50, 1])

        ax1 = plt.subplot(gs[0, 0])
        ax2 = plt.subplot(gs[1, 0], sharex=ax1, sharey=ax1)
        cbax1 = plt.subplot(gs[:, 1])

        for i, obs in enumerate(lists['observations']):
            color = [colors[i][:-1]]

            if i==0:
                ax1.scatter(processed['coadd']['wave'],
                         processed[obs]['flux_rebinned_stellarRF'] / processed[obs]['rescaling'],
                         c=color, s=2, alpha=0.2, label='observation')
            else:
                ax1.scatter(processed['coadd']['wave'],
                         processed[obs]['flux_rebinned_stellarRF'] / processed[obs]['rescaling'],
                         c=color, s=2, alpha=0.2)

            ax2.scatter(processed['coadd']['wave'],
                     processed[obs]['flux_rebinned_stellarRF_corrected'] / processed[obs]['rescaling'],
                     c=color, s=3, alpha=0.2)

        ax1.plot(processed['coadd']['wave'], processed['coadd']['rescaled'], c='k', lw=1, label='reference spectrum')
        ax2.plot(processed['coadd']['wave'], processed['coadd']['rescaled'], c='k', lw=1)


        ax1.set_xlim(processed['coadd']['wave'][0], processed['coadd']['wave'][-1])
        ax1.set_ylim(y_limits)
        ax2.set_ylim(y_limits)
        ax1.legend(loc=1)
        ax1.set_title('Night: {0:s} \n Input spectra'.format(night))
        ax2.set_title('Corrected spectra')
        ax2.set_xlabel('$\lambda$ [$\AA$]')

        sm = plt.cm.ScalarMappable(cmap=color_cmap, norm=color_norm)
        sm.set_array([])  # You have to set a dummy-array for this to work...
        cbar = plt.colorbar(sm, cax=cbax1)
        cbar.set_label('BJD - 2450000.0')
        fig.subplots_adjust(wspace=0.05, hspace=0.4)
        plt.show()


        """
        PLOT
        """

        fig = plt.figure(figsize=(12, 6))
        gs = GridSpec(1, 2, width_ratios=[50, 1])
        ax = plt.subplot(gs[0, 0])
        cbax1 = plt.subplot(gs[:, 1])

        for i, obs in enumerate(lists['observations']):
            if i == 0:
                offset = np.std(processed[obs]['ratio'][refraction[obs]['poly_flag']].flatten()) * 6
                average = np.average(processed[obs]['ratio'][refraction[obs]['poly_flag']].flatten())
                y_limits = [average-offset, average+offset]

            color = [colors[i][:-1]]

            for order in range(0, input_data[obs]['n_orders']):

                ax.scatter(input_data[obs]['wave'][refraction[obs]['poly_flag']],
                           processed[obs]['ratio'][refraction[obs]['poly_flag']] + offset*i,
                           s=1, c=color, alpha=0.50, zorder=2)

                ax.scatter(input_data[obs]['wave'][~refraction[obs]['poly_flag']],
                           processed[obs]['ratio'][~refraction[obs]['poly_flag']] + offset*i,
                           s=2, c='k', alpha=0.05, zorder=1)
                ax.plot(input_data[obs]['wave'][order, :],
                        refraction[obs]['polyfit_e2ds'][order, :] + offset*i,
                        c='k', lw=1, zorder=5)

        y_limits_offset = [min(y_limits[0] + offset * i, y_limits[0]),
                           max(y_limits[1] + offset * i, y_limits[1])]

        ax.set_ylim(y_limits_offset)
        ax.set_xlabel('$\lambda$ [$\AA$]')
        ax.legend(loc=3)
        ax.set_title('Night: {0:s} \n Fit of the ratio obs/master'.format(night))

        sm = plt.cm.ScalarMappable(cmap=color_cmap, norm=color_norm)
        sm.set_array([])  # You have to set a dummy-array for this to work...
        cbar = plt.colorbar(sm, cax=cbax1)
        cbar.set_label('BJD - 2450000.0')
        fig.subplots_adjust(wspace=0.05, hspace=0.4)
        plt.show()


        """ 
        PLOT: residuals of the fit
        """

        fig = plt.figure(figsize=(12, 6))
        gs = GridSpec(1, 2, width_ratios=[50, 1])
        ax = plt.subplot(gs[0, 0])
        cbax1 = plt.subplot(gs[:, 1])

        for i, obs in enumerate(lists['observations']):
            if i == 0:
                median = np.median(processed[obs]['residuals'][refraction[obs]['poly_flag']].flatten())
                offset = np.std(processed[obs]['residuals'][refraction[obs]['poly_flag']].flatten()) * 6
                y_limits = [median-offset, median+offset]

            color = colors[i][:-1]

            for order in range(0, input_data[obs]['n_orders']):

                # Workaround to damn stupid matplotlib error I didn't manage to solve
                ax.scatter(input_data[obs]['wave'][refraction[obs]['poly_flag']],
                           processed[obs]['residuals'][refraction[obs]['poly_flag']] + offset*i,
                           s=1, c=[color], alpha=0.50, zorder=2)

                ax.scatter(input_data[obs]['wave'][~refraction[obs]['poly_flag']],
                           processed[obs]['residuals'][~refraction[obs]['poly_flag']] + offset*i,
                           s=2, c='k', alpha=0.05, zorder=1)

            ax.axhline(offset*i, c='k', zorder=3)

        y_limits_offset = [min(y_limits[0] + offset * i, y_limits[0]),
                           max(y_limits[1] + offset * i, y_limits[1])]

        ax.set_ylim(y_limits_offset)
        ax.set_xlabel('$\lambda$ [$\AA$]')
        ax.legend(loc=3)
        ax.set_title('Night: {0:s} \n Residuals of the fit on ratio obs/master'.format(night))

        sm = plt.cm.ScalarMappable(cmap=color_cmap, norm=color_norm)
        sm.set_array([])  # You have to set a dummy-array for this to work...
        cbar = plt.colorbar(sm, cax=cbax1)
        cbar.set_label('BJD - 2450000.0')
        fig.subplots_adjust(wspace=0.05, hspace=0.4)
        plt.show()

        """ 
        PLOT: corrected e2ds
        """

        fig = plt.figure(figsize=(12, 6))
        gs = GridSpec(1, 2, width_ratios=[50, 1])
        ax = plt.subplot(gs[0, 0])
        cbax1 = plt.subplot(gs[:, 1])

        for i, obs in enumerate(lists['observations']):
            color = [colors[i][:-1]]

            ax.scatter(input_data[obs]['wave'][refraction[obs]['poly_flag']],
                       processed[obs]['e2ds_corrected'][refraction[obs]['poly_flag']]/processed[obs]['rescaling'],
                       s=2, c=color, alpha=0.10)

            ax.scatter(input_data[obs]['wave'][~refraction[obs]['poly_flag']],
                       processed[obs]['e2ds_corrected'][~refraction[obs]['poly_flag']]/processed[obs]['rescaling'],
                       s=2, c='k', alpha=0.05)

            #for order in range(0, np.size(input_data[obs]['wave'][:, 0])):
            #
            #    ax.plot(input_data[obs]['wave'][order, :],
            #            refraction[obs]['polyfit_e2ds'][order, :],
            #            c=color_array, lw=1)

        ax.set_xlabel('$\lambda$ [$\AA$]')
        ax.set_title('Night: {0:s} \n Corrected and rescaled e2ds spectra'.format(night))

        sm = plt.cm.ScalarMappable(cmap=color_cmap, norm=color_norm)
        sm.set_array([])  # You have to set a dummy-array for this to work...
        cbar = plt.colorbar(sm, cax=cbax1)
        cbar.set_label('BJD - 2450000.0')
        fig.subplots_adjust(wspace=0.05, hspace=0.4)
        plt.show()


from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.spectral_subroutines import *
from SLOPpy.subroutines.io_subroutines import *
from SLOPpy.subroutines.fit_subroutines import *
from SLOPpy.subroutines.plot_subroutines import *
from SLOPpy.subroutines.shortcuts import *
from SLOPpy.subroutines.eso_skycalc_cli import get_eso_sckycalc_harps

__all__ = ["compute_telluric_observerRF_skycalc", "plot_telluric_observerRF_skycalc"]


def compute_telluric_observerRF_skycalc(config_in):

    night_dict = from_config_get_nights(config_in)

    for night in night_dict:

        print()
        print("compute_telluric_airmass_observerRF        Night: ", night)

        try:
            telluric = load_from_cpickle('telluric', config_in['output'], night)
            continue
        except:
            print("No telluric correction file found, computing now ")
            print()

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)

        """ Retrieving the observations"""
        calib_data = load_from_cpickle('calibration_fibA', config_in['output'], night)
        input_data = retrieve_observations(config_in['output'], night, lists['observations'], use_telluric=False)
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        processed = {
            'subroutine': 'telluric_observerRF_skycalc',
            'n_orders': 0,
            'n_pixels': 0
        }

        telluric = {
            'subroutine': 'telluric_observerRF_skycalc',
            'reference_frame': 'observer'
        }

        # There must be a more elegant way to do this, but I'm, not aware of it
        for obs in lists['observations']:

            processed[obs] = {
                'n_orders': input_data[obs]['n_orders'],
                'n_pixels': input_data[obs]['n_pixels']
            }

            """ for plotting purpose only"""
            processed[obs]['wave'] = input_data[obs]['wave']
            processed[obs]['e2ds'] = input_data[obs]['e2ds']
            processed[obs]['e2ds_err'] = input_data[obs]['e2ds_err']

            processed[obs]['e2ds_rescaling'], processed[obs]['e2ds_rescaled'], processed[obs]['e2ds_rescaled_err'] = \
                perform_rescaling(input_data[obs]['wave'],
                                  input_data[obs]['e2ds'],
                                  input_data[obs]['e2ds_err'],
                                  observational_pams['wavelength_rescaling'])

            if processed['n_orders'] == 0:
                processed['n_orders'] = input_data[obs]['orders']
                processed['n_pixels'] = input_data[obs]['wave_size']

        """ Reference airmass for iterative correction of airmass"""

        telluric['sky_template'] = {}
        instrument = night_dict[night]['instrument']

        for n_obs, obs in enumerate(lists['transit_in']):
            if n_obs >= lists['n_transit_in']/2.:
                obs_ref = obs
                break

        telluric['sky_template']['ref_observation'] = obs_ref
        if instrument == 'HARPS':
            telluric['sky_template']['use_eso_skycalc'] = True
            telluric['sky_template']['ref_airmass'] = input_data[obs_ref]['AIRMASS']
            telluric['sky_template']['data'] = np.ones([processed['n_orders'], processed['n_pixels']], dtype=np.double)

        else:
            telluric_model_file = config_in['instruments'][instrument]['telluric_model']
            telluric['sky_template']['use_eso_skycalc'] = False
            telluric['sky_template']['ref_airmass'] = 1.00000
            telluric['sky_template']['data'] = fits.open(telluric_model_file)

        for obs in lists['observations']:
            processed[obs]['e2ds_precorrected'] = processed[obs]['e2ds_rescaled'][:]
            processed[obs]['e2ds_precorrected_err'] = input_data[obs]['e2ds_err'] / processed[obs]['e2ds_rescaling']

        for niter in xrange(0, 1):
            print("NITER:            ", niter)

            for obs in lists['telluric']:
                processed[obs]['logI'] = np.log(processed[obs]['e2ds_precorrected'])
                processed[obs]['logI_err'] = processed[obs]['e2ds_precorrected_err']/processed[obs]['e2ds_precorrected']

            processed['telluric'] = {}

            abs_slope = np.ones([processed['n_orders'], processed['n_pixels']], dtype=np.double)
            line_shift = np.ones([processed['n_orders'], processed['n_pixels']], dtype=np.double)
            zero_point = np.ones([processed['n_orders'], processed['n_pixels']], dtype=np.double)
            pearson_r = np.zeros([processed['n_orders'], processed['n_pixels']], dtype=np.double)
            pearson_p = np.zeros([processed['n_orders'], processed['n_pixels']], dtype=np.double)

            airmass = np.zeros(lists['n_tellurics'], dtype=np.double)
            berv = np.zeros(lists['n_tellurics'], dtype=np.double)
            rvc = np.zeros(lists['n_tellurics'], dtype=np.double)

            for n_obs, obs in enumerate(lists['telluric']):
                # This is to ensure that airmass, berv and rvc are associated to the correct spectra
                processed['telluric'][obs] = {'n_obs': n_obs}
                airmass[n_obs] = input_data[obs]['AIRMASS']
                berv[n_obs] = input_data[obs]['BERV']
                rvc[n_obs] = input_data[obs]['RVC']

            for order in range(0, processed['n_orders']):

                logi_array = np.empty([lists['n_tellurics'], processed['n_pixels']], dtype=np.double)
                sigi_array = np.empty([lists['n_tellurics'], processed['n_pixels']], dtype=np.double)

                for obs in lists['telluric']:
                    n_obs = processed['telluric'][obs]['n_obs']
                    logi_array[n_obs, :] = processed[obs]['logI'][order, :]
                    sigi_array[n_obs, :] = processed[obs]['logI_err'][order, :]

                """ The user has the option to select between different approaches to
                    extract the telluric absorption spectrum
                    To-Do: move this section to a subroutine for cythonization"""

                if observational_pams['linear_fit_method']== 'linear_curve_fit':

                    abs_slope[order, :], line_shift[order, :], zero_point[order, :] = \
                        berv_linear_curve_fit_modified(airmass, berv, logi_array, sigi_array, processed['n_pixels'])

                else:
                    abs_slope[order, :], line_shift[order, :], zero_point[order, :] = \
                        berv_linear_lstsq(airmass, berv, logi_array)

                """ Saving the outcome to dictionary """
                processed['telluric']['order_'+repr(order)] = {'logi_array': logi_array, 'sigi_array': sigi_array}

                if telluric['sky_template']['use_eso_skycalc']:
                    wave_model, step_model, tran_model, terr_model = \
                        get_eso_sckycalc_harps(obs_ref,
                                               [input_data[obs_ref]['wave'][0], input_data[obs_ref]['wave'][-1]],
                                               input_data[obs_ref]['RA'],
                                               input_data[obs_ref]['DEC'],
                                               night, config_in['output'])

                    tran_model_rebinned = \
                        rebin_1d_to_1d(wave_model,
                                       step_model,
                                       tran_model,
                                       input_data[obs_ref]['wave'],
                                       input_data[obs_ref]['step'],
                                       preserve_flux=False)

                    telluric['sky_template']['data'][order, :] = \
                        np.power(tran_model_rebinned,
                                 1./telluric['sky_template']['ref_airmass'])

            processed['telluric']['spectrum_noairmass'] = np.exp(abs_slope)

            for obs in lists['observations']:
                """ Correction of telluric lines for the average airmass value, following Wyttenbach et al. 2015 """
                processed[obs]['e2ds_corrected'] = processed[obs]['e2ds_precorrected'] / \
                                                        np.power(processed['telluric']['spectrum_noairmass'],
                                                                 input_data[obs]['AIRMASS'] -
                                                                 processed['airmass_ref'])
                processed[obs]['e2ds_corrected_err'] = processed[obs]['e2ds_precorrected_err'] / \
                                                        np.power(processed['telluric']['spectrum_noairmass'],
                                                                 input_data[obs]['AIRMASS'] -
                                                                 processed['airmass_ref'])

        for obs in lists['observations']:
            # Correction of telluric lines

            telluric[obs] = {}

            telluric[obs]['spectrum_noairmass'] = processed['telluric']['spectrum_noairmass']

            telluric[obs]['airmass'] = input_data[obs]['AIRMASS']
            telluric[obs]['airmass_ref'] = processed['airmass_ref']
            telluric[obs]['null'] = telluric[obs]['spectrum_noairmass'] < 0.001
            telluric[obs]['spectrum_noairmass'][ telluric[obs]['null']] = 1.0

            telluric[obs]['spectrum'] = np.power(processed['telluric']['spectrum_noairmass'],
                                       input_data[obs]['AIRMASS'] - processed['airmass_ref'])

            telluric[obs]['spline_noairmass'] = np.ones([input_data[obs]['n_orders'],
                                                         input_data[obs]['n_pixels']],
                                                        dtype=np.double)

            for order in range(0, processed['n_orders']):
                telluric[obs]['spline_noairmass'][order, :], _, _ = \
                    compute_spline(input_data[obs]['wave'][order, :],
                                   telluric[obs]['spectrum_noairmass'][order, :],
                                   0.05)

            telluric[obs]['spline'] = np.power(telluric[obs]['spline_noairmass'],
                                       input_data[obs]['AIRMASS'] - processed['airmass_ref'])

            telluric[obs]['airmass'] = input_data[obs]['AIRMASS']
            telluric[obs]['airmass_ref'] = processed['airmass_ref']
            telluric[obs]['null'] = telluric[obs]['spectrum_noairmass'] < 0.001
            telluric[obs]['spectrum_noairmass'][ telluric[obs]['null']] = 1.0

            telluric[obs]['telluric_corrected'] = processed[obs]['e2ds_corrected']
            telluric[obs]['telluric_corrected_err'] = processed[obs]['e2ds_corrected_err']

        save_to_cpickle('telluric', telluric, config_in['output'], night)
        save_to_cpickle('telluric_processed', processed, config_in['output'], night)

        print()
        print("Night ", night, " completed")


def plot_telluric_observerRF_skycalc(config_in, night_input=''):
    import matplotlib.pyplot as plt

    night_dict = from_config_get_nights(config_in)
    instrument_dict = from_config_get_instrument(config_in)
    system_dict = from_config_get_system(config_in)

    if night_input=='':
        night_list = night_dict
    else:
        night_list = np.atleast_1d(night_input)

    for night in night_list:

        print("plot_telluric_airmass_stellarRF            Night: ", night)

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        """ Retrieving the analysis"""
        try:
            processed = load_from_cpickle('telluric_processed', config_in['output'], night)
            telluric = load_from_cpickle('telluric', config_in['output'], night)
        except:
            print()
            print("No telluric correction, no plots")
            continue

        colors, cmap, line_colors = make_color_array(lists, observational_pams)

        fig = plt.figure(figsize=(12, 6))
        gs = GridSpec(2, 2, width_ratios=[50, 1])

        ax1 = plt.subplot(gs[0, 0])
        ax2 = plt.subplot(gs[1, 0], sharex=ax1)
        cbax1 = plt.subplot(gs[:, 1])

        lift_spectrum = 0.25

        for i, obs in enumerate(lists['observations']):

            _, e2ds_rescaled , _ = \
                perform_rescaling(processed[obs]['wave'],
                                  processed[obs]['e2ds'],
                                  processed[obs]['e2ds_err'],
                                  observational_pams['wavelength_rescaling'])

            e2ds_rescaled_corrected_spectrum = e2ds_rescaled / telluric[obs]['spectrum']

            for order in range(0, processed[obs]['n_orders']):

                ax1.plot(processed[obs]['wave'][order, :],
                            e2ds_rescaled[order, :],
                            c=line_colors[i], lw=1, alpha=0.5)
                ax1.scatter(processed[obs]['wave'][order, :],
                            e2ds_rescaled_corrected_spectrum[order, :],
                            s=1, c=line_colors[i])

                ax2.plot(processed[obs]['wave'][order, :],
                         telluric[obs]['spectrum'][order, :],
                         c=line_colors[i])
                ax2.axhline(1.00, c='k')


        ax1.legend(loc=3)
        ax1.set_title('Night: ' + night)

        ax2.set_xlabel('$\lambda$ [$\AA$]')

        obs_ref = telluric['sky_template']['ref_observation']

        for order in range(0, processed[obs]['n_orders']):
            ax2.scatter(processed[obs_ref]['wave'][order, :], telluric[obs_ref]['spectrum_noairmass'][order, :], s=2, c='C0', zorder=1000)
            ax2.plot(processed[obs_ref]['wave'][order, :], telluric['sky_template']['data'][order, :], c='C0', zorder=1000)
            ax1.plot(processed[obs_ref]['wave'][order, :], telluric['sky_template']['data'][order, :], c='C0', zorder=1000)

        sm = plt.cm.ScalarMappable(cmap=cmap, norm=plt.Normalize(vmin=colors[0], vmax=colors[-1]))
        sm.set_array([])  # You have to set a dummy-array for this to work...
        cbar = plt.colorbar(sm, cax=cbax1)
        cbar.set_label('BJD - 2450000.0')
        fig.subplots_adjust(wspace=0.05, hspace=0.4)
        plt.show()


from __future__ import print_function, division

from SLOPpy.instruments.common_DRSv3 import *

def HARPSv3_get_instrument_keywords():
    """ These definitions applt to DRS version 3.x """
    keywords = {
        'header_rvc': 'HIERARCH ESO DRS CCF RVC',
        'header_berv': 'HIERARCH ESO DRS BERV',
        'header_bjd': 'HIERARCH ESO DRS BJD',
        'header_mjd': 'MJD-OBS', # MJD in days. This parameter is required for the retrieval of GDAS data

        'header_blaze': 'HIERARCH ESO DRS BLAZE FILE',
        'header_ccd': 'HIERARCH ESO DRS CCD SIGDET',
        'header_conad': 'HIERARCH ESO DRS CCD CONAD',

        'header_dpr_catg': 'HIERARCH ESO DPR CATG',
        'header_dpr_type': 'HIERARCH ESO DPR TYPE',

        'header_deg_ll': 'HIERARCH ESO DRS CAL TH DEG LL',
        'header_coeff_ll': 'HIERARCH ESO DRS CAL TH COEFF LL',

        'airmass_alt_start': 'HIERARCH ESO TEL AIRM START',
        'airmass_alt_end': 'HIERARCH ESO TEL AIRM END',

        ## Telescope altitude is computed using the middle values obtained from airmass
        'humidity':'HIERARCH ESO TEL AMBI RHUM', # Relative humidity in % for GEOELEV.
        #'pressure_start' : 'HIERARCH ESO TEL AMBI PRES START',
        #'pressure_end': 'HIERARCH ESO TEL AMBI PRES END',
        'pressure':'HIERARCH ESO TEL AMBI PRES END',
        'temperature_env': 'HIERARCH ESO TEL AMBI TEMP', #Ambient temperature in C for GEOELEV
        'temperature_m1': 'HIERARCH ESO TEL TH M1 TEMP', # Temperature of primary mirror M1 in C (for emission spectra only)

    }

    properties = {
        # DRS-specific keywords
        'time_stamp': 'mid_exposure',
        'time_standard': 'UTC',

        # Observatory-specific keywords
        'geoelev': 2400.0, # meters
        'longitude' : -70.7345, # Tel geo longitude (+=East) (deg)
        'latitude' : -29.2584,  # Tel geo latitute (+=North) (deg)
        # Instrument-specific keyword
        'n_orders_A': 72,
        'n_orders_B': 71,
        'orders_BtoA':
            [ 0,  1,  2,  3,  4,  5,  6,  7,  8,  9,
                10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
                20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
                30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
                40, 41, 42, 43, 44, -1, 45, 46, 47, 48,
                49, 50, 51, 52, 53, 54, 55, 56, 57, 58,
                59, 60, 61, 62, 63, 64, 65, 66, 67, 68,
                69, 70],
        # after many experiments, I found out the easiest and more robust way to define
        # the order correspondence between fiber A anf B is just to write it down
        'red_ccd':
            [                    45, 46, 47, 48, 49,
                50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
                60, 61, 62, 63, 64, 65, 66, 67, 68, 69,
                70, 71],
        'blue_ccd':
            [0,  1,  2,  3,  4,  5,  6,  7,  8,  9,
                10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
                20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
                30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
                40, 41, 42, 43, 44],
        'full_ccd':
            [0,  1,  2,  3,  4,  5,  6,  7,  8,  9,
                10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
                20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
                30, 31, 32, 33, 34, 35, 36, 37, 38, 39,
                40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
                50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
                60, 61, 62, 63, 64, 65, 66, 67, 68, 69,
                70, 71],
        # The following are the input values used by Molecfit, taken from Allart+2017
        # for convenience, all the default values are listed here instead of being scattered into the code
        'molecfit': {
            'default_wstep': 0.01000, # default wavelength step size for the input stellar spectra
            'molecules': ['H2O', 'O2'],
            'ftol': 1e-9,
            'xtol': 1e-9,

            'cont_const': 1.0, # a0,  This value differs from Allart+2017 since we are using normalized spectra
            'cont_n': 3, # n_cont, Degree of coefficients for continuum fit
            'wlc_n': 2, # n_lambda, Polynomial degree of the refined wavelength solution
            'wlc_const': 0.0, # b0, Initial constant term for wavelength correction (shift relative to half  wavelength range)
            'res_gauss': 4.8, # omega_Gaussian,  Initial value for FWHM of Gaussian in pixels
            'kernfac': 15, #kernel_size, Size of Gaussian/Lorentzian/Voigtian kernel in FWHM

            'slitwidth': 1.00, # in arcseconds
            'pixelscale': 0.16,
        }
    }

    return keywords, properties

# Shortcut from DRS-geenral to instrument-specific subroutine
def HARPS_DRSv3_get_calib_data(archive, file_rad, fiber='A', order_selection=None):

    keywords, properties = HARPSv3_get_instrument_keywords()
    return DRSv3_get_calib_data(archive, file_rad, keywords, properties, fiber=fiber, order_selection=order_selection)

# Shortcut from DRS-geenral to instrument-specific subroutine
def HARPS_DRSv3_get_input_data(archive, file_rad, mask, fiber='A', skip_ccf=None, skip_s1d=True, order_selection=None):

    keywords, properties = HARPSv3_get_instrument_keywords()
    return DRSv3_get_input_data(archive, file_rad, keywords, properties, mask, fiber=fiber, skip_ccf=skip_ccf, skip_s1d=skip_s1d, order_selection=order_selection)



from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.spectral_subroutines import *
from SLOPpy.subroutines.io_subroutines import *
from SLOPpy.subroutines.fit_subroutines import *
from SLOPpy.subroutines.shortcuts import *

__all__ = [
           'compute_transmission_spectrum_average',
           'plot_transmission_spectrum_average'
           ]


subroutine_name = 'transmission_spectrum_average'
pick_files =  'transmission_spectrum'
sampler = 'emcee'


def compute_transmission_spectrum_average(config_in, lines_label, reference='planetRF', pca_iteration=-1):

    night_dict = from_config_get_nights(config_in)

    spectral_lines = from_config_get_spectral_lines(config_in)
    line_iter_dict = spectral_lines[lines_label]

    shared_data = load_from_cpickle('shared', config_in['output'])

    total_n_transit_full = 0
    total_n_transit_out = 0

    total_lists = {}

    """ Using the user defined range to define the transmission spectrum region 
        This range can be larger than the one defined for the MCMC range, and it
        MUST include the continuum windws for the transmission lightcurve
    """

    shared_selection = (shared_data['coadd']['wave'] >= line_iter_dict['range'][0]) \
        & (shared_data['coadd']['wave'] < line_iter_dict['range'][1])
    binned_selection = (shared_data['binned']['wave'] >= line_iter_dict['range'][0]) \
        & (shared_data['binned']['wave'] < line_iter_dict['range'][1])

    transmission_average_template = {
        'subroutine': subroutine_name + '_' + reference,
        'range': line_iter_dict['range'],
        'wave': shared_data['coadd']['wave'][shared_selection],
        'step': shared_data['coadd']['step'][shared_selection],
        'size': int(np.sum(shared_selection)),
        'binned_wave': shared_data['binned']['wave'][binned_selection],
        'binned_step': shared_data['binned']['step'][binned_selection],
        'binned_size': int(np.sum(binned_selection))
    }

    results_list = ['user',
                    'mcmc_night_MED',
                    'mcmc_night_MAP',
                    'mcmc_global_MED',
                    'mcmc_global_MAP']

    for results_selection in results_list:

        """ First check to see if we need to compute the average transmission
        iteratively when PCA has been employed """

        for night in night_dict:
            preparation_input = load_from_cpickle('transmission_preparation', config_in['output'], night)
            if preparation_input.get('pca_output', False):
                if pca_iteration >= 0:
                    it_string = str(pca_iteration).zfill(2)
                else:
                    it_string = str(pca_parameters.get('ref_iteration')).zfill(2)
            else:
                it_string = ''
            preparation = None
            break


        transmission_average = transmission_average_template.copy()

        try:
            transmission_average = load_from_cpickle(subroutine_name + '_' + reference + '_' + results_selection, config_in['output'], lines=lines_label, it_string=it_string)
            print("{0:45s}    {1:s}   {2:s}".format(subroutine_name + '_' + reference, results_selection, 'Retrieved'))
            continue
        except (FileNotFoundError, IOError):
            skip_iteration = False
            #print("{0:45s}    {1:s}   {2:s}".format(subroutine_name + '_' + reference, results_selection, 'Computing'))


        #skip_iteration = False
        for night in night_dict:
            # """ Retrieving the list of observations"""

            total_lists[night] = load_from_cpickle('lists', config_in['output'], night)
            #try:
            #    transmission_average[night] = load_from_cpickle('transmission_second_telluric_'+reference, config_in['output'], night)
            #    print("   Using transmission spectra with second telluric correction for Night: {0:s}".format(night))
            #except:
            #    transmission_average[night] = load_from_cpickle('transmission_'+reference, config_in['output'], night)

            try:
                transmission_average[night] = load_from_cpickle(pick_files + '_' + reference + '_' + results_selection, config_in['output'], night, lines_label, it_string)
            except:
                skip_iteration = True
                print(skip_iteration)
            total_n_transit_full += len(total_lists[night]['transit_full'])
            total_n_transit_out += len(total_lists[night]['transit_out'])

        if skip_iteration: continue
        print("{0:45s}    {1:s}   {2:s}".format(subroutine_name + '_' + reference, results_selection, 'Computing'))

        array_average_in = np.zeros([total_n_transit_full, transmission_average['size']])
        weights_average_in = np.zeros([total_n_transit_full, transmission_average['size']])
        clvrm_average_in = np.zeros([total_n_transit_full, transmission_average['size']])
        uncorr_average_in = np.zeros([total_n_transit_full, transmission_average['size']])
        i_total_in = 0

        array_average_out = np.zeros([total_n_transit_out, transmission_average['size']])
        weights_average_out = np.zeros([total_n_transit_out, transmission_average['size']])
        i_total_out = 0

        for night in night_dict:

            for obs in total_lists[night]['transit_full']:
                array_average_in[i_total_in, :] = transmission_average[night][obs]['normalized'][:]
                weights_average_in[i_total_in, :] = 1./(transmission_average[night][obs]['normalized_err']**2.)
                clvrm_average_in[i_total_in, :] = transmission_average[night][obs]['clv_model_rebinned'][:]
                uncorr_average_in[i_total_in, :] = transmission_average[night][obs]['normalized_uncorrected'][:]
                i_total_in += 1

            for obs in total_lists[night]['transit_out']:
                array_average_out[i_total_out, :] = transmission_average[night][obs]['normalized'][:]
                weights_average_out[i_total_out, :] = 1. / (transmission_average[night][obs]['normalized_err'] ** 2.)
                i_total_out += 1

        transmission_average['average'], transmission_average['sum_weights'] = np.average(
            array_average_in, axis = 0, weights = weights_average_in, returned = True)
        transmission_average['average_err'] = 1./np.sqrt(transmission_average['sum_weights'])

        transmission_average['average_clv_model'], _ = np.average(
            clvrm_average_in, axis = 0, weights = weights_average_in, returned = True)

        transmission_average['average_uncorrected'], _ = np.average(
            uncorr_average_in, axis = 0, weights = weights_average_in, returned = True)

        transmission_average['binned'] = \
            rebin_1d_to_1d(transmission_average['wave'],
                        transmission_average['step'],
                        transmission_average['average'],
                        transmission_average['binned_wave'],
                        transmission_average['binned_step'],
                        preserve_flux=False)
        transmission_average['binned_err'] = \
            rebin_1d_to_1d(transmission_average['wave'],
                        transmission_average['step'],
                        transmission_average['average_err'],
                        transmission_average['binned_wave'],
                        transmission_average['binned_step'],
                        preserve_flux=False,
                        is_error=True)

        transmission_average['binned_clv_model'] = \
            rebin_1d_to_1d(transmission_average['wave'],
                        transmission_average['step'],
                        transmission_average['average_clv_model'],
                        transmission_average['binned_wave'],
                        transmission_average['binned_step'],
                        preserve_flux=False)

        transmission_average['binned_uncorrected'] = \
            rebin_1d_to_1d(transmission_average['wave'],
                        transmission_average['step'],
                        transmission_average['average_uncorrected'],
                        transmission_average['binned_wave'],
                        transmission_average['binned_step'],
                        preserve_flux=False)

        transmission_average['average_out'], transmission_average['sum_weights_out'] = np.average(
            array_average_out, axis=0, weights=weights_average_out, returned=True)
        transmission_average['average_out_err'] = 1./np.sqrt(transmission_average['sum_weights_out'])

        transmission_average['binned_out'] = \
            rebin_1d_to_1d(transmission_average['wave'],
                        transmission_average['step'],
                        transmission_average['average_out'],
                        transmission_average['binned_wave'],
                        transmission_average['binned_step'],
                        preserve_flux=False)
        transmission_average['binned_out_err'] = \
            rebin_1d_to_1d(transmission_average['wave'],
                        transmission_average['step'],
                        transmission_average['average_out_err'],
                        transmission_average['binned_wave'],
                        transmission_average['binned_step'],
                        preserve_flux=False,
                        is_error=True)

        save_to_cpickle(subroutine_name + '_' + reference + '_' + results_selection, transmission_average, config_in['output'], lines=lines_label, it_string=it_string)


def plot_transmission_spectrum_average(config_in, lines_label, night_input='', results_input='', reference='planetRF', pca_iteration=-1):

    spectral_lines = from_config_get_spectral_lines(config_in)
    lines_dict = spectral_lines[lines_label]

    if results_input=='':
        results_list = ['user',
                'mcmc_night_MED',
                'mcmc_night_MAP',
                'mcmc_global_MED',
                'mcmc_global_MAP']
    else:
        results_list = np.atleast_1d(results_input)

    os.system('mkdir -p plots')

    interactive_plots = from_config_get_interactive_plots(config_in)



    # Workaround to check if the transmission spectrum has been obtained through PCA iterations
    night_dict = from_config_get_nights(config_in)
    for night in night_dict:
        preparation_input = load_from_cpickle('transmission_preparation', config_in['output'], night)

        if preparation_input.get('pca_output', False):
            if pca_iteration >= 0:
                it_string = str(pca_iteration).zfill(2)
            else:
                it_string = str(preparation_input.get('ref_iteration')).zfill(2)
        else:
            it_string = ''
        preparation_input = None
        break



    for results_selection in results_list:

        try:
            transmission_average = load_from_cpickle(subroutine_name + '_' + reference + '_' + results_selection, config_in['output'], lines=lines_label, it_string=it_string)
            print("{0:45s}    {1:s}   {2:s}".format(subroutine_name + '_' + reference, results_selection, 'Plotting'))
        except (FileNotFoundError, IOError):
            print("{0:45s}                         {1:s}".format(subroutine_name + '_' + reference, 'Plot skipped'))
            return

        filename_rad = subroutine_name + '_' + reference + '_' + results_selection

        fig = plt.figure(figsize=(12, 9))

        gs = GridSpec(2, 1)
        ax1 = plt.subplot(gs[0, 0])
        ax2 = plt.subplot(gs[1, 0], sharex=ax1, sharey=ax1)

        spec_offset = 0.025

        ax1.errorbar(transmission_average['wave'],
                    transmission_average['average'],
                    yerr=transmission_average['average_err'],
                    fmt='ko', ms=1, zorder=10, alpha=0.10, label='average')

        #ax1.scatter(transmission_average['wave'],
        #            transmission_average['average'],
        #            c='black',
        #            s=2, zorder=15,
        #            label='average',
        #            )

        ax1.errorbar(transmission_average['binned_wave'],
                    transmission_average['binned'],
                    yerr=transmission_average['binned_err'],
                    fmt='ko', ms=3, zorder=20, label='binned')

        #ax1.scatter(transmission_average['wave'],
        #            transmission_average['average'],
        #            c='black',
        #            s=2, zorder=200,
        #            label='average',
        #            )

        ax2.errorbar(transmission_average['binned_wave'],
                    transmission_average['binned_out'],
                    yerr=transmission_average['binned_out_err'],
                    fmt='ko', ms=3, zorder=20, label='binned out')

        ax2.errorbar(transmission_average['wave'],
                    transmission_average['average_out'],
                    yerr=transmission_average['average_out_err'],
                    fmt='ko', ms=1, zorder=10, alpha=0.10, label='average out')

        for n_night, night in enumerate(night_dict):
            ax1.errorbar(transmission_average['wave'],
                        transmission_average[night]['average']-spec_offset*(1.+n_night),
                        yerr=transmission_average[night]['average_err'],
                        color='C'+repr(n_night),
                        fmt='o', ms=1, zorder=1, alpha=0.25)

            ax1.scatter(transmission_average['wave'],
                        transmission_average[night]['average']-spec_offset*(1.+n_night),
                        c='C'+repr(n_night),
                        s=2, zorder=2,
                        label=night,
                        )

            ax2.errorbar(transmission_average['wave'],
                        transmission_average[night]['average_out']-spec_offset*(1.+n_night),
                        yerr=transmission_average[night]['average_out_err'],
                        color='C'+repr(n_night),
                        fmt='o', ms=1, zorder=1, alpha=0.25)

            ax2.scatter(transmission_average['wave'],
                        transmission_average[night]['average_out']-spec_offset*(1.+n_night),
                        c='C'+repr(n_night),
                        s=2, zorder=2)

        #ax1.set_ylim(0.95-spec_offset*(1.+n_night), 1.05)
        #ax1.set_xlim(config_in['master-out']['wavelength_range'][0], config_in['master-out']['wavelength_range'][1])
        try:
            ax1.set_xlim(lines_dict['plot_range'][0], lines_dict['plot_range'][1])
        except:
            ax1.set_xlim(lines_dict['range'][0], lines_dict['range'][1])

        ax1.set_ylim(0.985, 1.01)
        ax2.set_xlabel('$\lambda$ [$\AA$]')
        ax1.legend(loc=3)
        ax1.set_title('Average in-transit transmission spectrum in {0:s}'.format(reference))
        ax2.set_title('Average out-transit transmission spectrum in {0:s}'.format(reference))

        output_file = get_filename(filename_rad + '_binned', config_in['output'], night='', lines=lines_label, extension='.pdf', it_string=it_string)
        plt.savefig('plots/'+output_file, bbox_inches='tight', dpi=300)
        if interactive_plots:
            plt.show()
        plt.close()



from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.io_subroutines import get_filename


def get_eso_sckycalc_harps(obs_ref, wave_range, ra, dec, night, output):

    # https://www.eso.org/observing/etc/doc/skycalc/helpskycalccli.html

    time_tag = obs_ref[6:25]
    wave_range_nm = wave_range/10.
    wdelta = 0.00075

    input_filename = get_filename('skycalc_input_' + repr(wave_range_nm[0]) + '_' + repr(wave_range_nm[1]),
                                  output, night, extension=".JSON")

    alman_filename = get_filename('skycalc_alman_' + time_tag + '_' + repr(wave_range_nm[1]),
                                  output, night, extension=".JSON")

    output_filename = get_filename('skycalc_output_' + repr(wave_range_nm[0]) + '_' + repr(wave_range_nm[1]) + time_tag,
                                  output, night, extension=".fits")

    if os.path.isfile(output_filename):
        skycalc_hdu = fits.open(night + '.fits')
        data = skycalc_hdu[1].data

        skycalc_hdu.close()
        return data.field(0) * 10., \
               np.ones(len(data.field(0))) * wdelta, \
               data.field(14), \
               np.ones(len(data.field(0)))

    if not os.path.isfile(input_filename):
        input_pams = {
            "pwv_mode": "pwv",
            "incl_moon": "N", # No moon contamination
            "incl_starlight": "N", # No starlight
            "incl_zodiacal": "N", # No zodiacal light
            "incl_loweratm": "Y",
            "incl_upperatm": "Y",
            "incl_airglow": "N", # No airglow
            "incl_therm": "N",
            "vacair": "air", # compute in the air
            "wmin": wave_range[0],
            "wmax": wave_range[1],
            "wgrid_mode": "fixed_wavelength_step",
            "wdelta": wdelta,
            "wres": 20000,
            "lsf_type": "Gaussian", # Gaussian lsf
            "lsf_gauss_fwhm": 5.5,
        }

        with open(input_filename, 'w') as outfile:
            json.dump(input_pams, outfile)

    if not os.path.isfile(alman_filename):
        almanac_pams = {
            "ra": ra,
            "dec": dec,
            "date": time_tag,
            "observatory": "lasilla"
        }

        with open(alman_filename, 'w') as outfile:
            json.dump(almanac_pams, outfile)

    os.system('skycalc_cli'
               + ' --in ' + input_filename
               + ' --alm ' + alman_filename
               + ' --out ' + output_filename)

    skycalc_hdu = fits.open(output_filename)
    data = skycalc_hdu[1].data

    skycalc_hdu.close()

    return data.field(0) * 10., \
           np.ones(len(data.field(0))) * wdelta, \
           data.field(14), \
           np.ones(len(data.field(0)))



from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.spectral_subroutines import *
from SLOPpy.subroutines.fit_subroutines import *
from SLOPpy.subroutines.io_subroutines import *
from SLOPpy.subroutines.shortcuts import *
from SLOPpy.subroutines.plot_subroutines import *
from SLOPpy.differential_refraction_preparation import compute_differential_refraction_preparation

__all__ = ["compute_differential_refraction",
           "plot_differential_refraction",
           "compute_differential_refraction_update",
           "plot_differential_refraction_update"]


def compute_differential_refraction_update(config_in):
    compute_differential_refraction(config_in, append_name='update')


def plot_differential_refraction_update(config_in, night_input=''):
    plot_differential_refraction(config_in, night_input, append_name='update')


def compute_differential_refraction(config_in, append_name=None):

    if append_name:
        subroutine_name = 'differential_refraction_' + append_name
        filename = 'refraction_' + append_name
    else:
        subroutine_name = 'differential_refraction'
        filename = 'refraction'

    compute_differential_refraction_preparation(config_in, append_name)

    night_dict = from_config_get_nights(config_in)

    for night in night_dict:

        try:
            refraction = load_from_cpickle(filename, config_in['output'], night)
            print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Retrieved'))
            continue
        except:
            print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Computing'))
            print()

        refraction_dict = from_config_refraction(config_in, night)

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)

        """ Retrieving input and calibration data """
        calib_data = load_from_cpickle('calibration_fibA', config_in['output'], night)
        input_data = retrieve_observations(config_in['output'], night, lists['observations'],
                                           use_refraction=False)
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        preparation = load_from_cpickle(filename + '_preparation', config_in['output'], night)

        defined_reference = night_dict[night]['refraction'].get('reference', False)

        if defined_reference:
            reference = load_from_cpickle(filename + '_reference', config_in['output'])
            preparation['coadd']['wave'] = reference['wave']
            preparation['coadd']['step'] = reference['step']
            preparation['coadd']['rescaled'] = reference['rescaled']

            if not preparation.get('absolute_SRF', False):
                print("  Observations and reference spectra are in different reference system ")
                quit()

        processed = {
            'subroutine': subroutine_name,
            'coadd': {
                'wave': preparation['coadd']['wave'],
                'step': preparation['coadd']['step'],
                'size': preparation['coadd']['size']
            }
        }

        refraction = {
            'subroutine': subroutine_name,
            'wave': preparation['coadd']['wave'],
            'binned': {}
        }

        refraction['binned']['wave'] = np.arange(preparation['coadd']['wave'][0],
                                                 preparation['coadd']['wave'][-11],
                                                 20.*preparation['coadd']['step'][0], dtype=np.double)

        refraction['binned']['size'] = np.size(refraction['binned']['wave'])
        refraction['binned']['step'] = np.ones(refraction['binned']['size'], dtype=np.double) \
            * 20. * preparation['coadd']['step'][0]

        if refraction_dict['approach'] == 'full_spectrum':
            print("  Differential refraction performed over the full spectrum")
        elif refraction_dict['approach'] == 'individual_order':
            print("   Differential refraction performed order-by-order ")
        else:
            raise ValueError("ERROR: fitting approach for differential refraction not implemented")

        if refraction_dict['method'] == 'spline':
            print("  Modelling performed with spline")
            print("  Spline order for differential refraction fit: ",
                  refraction_dict['fit_order'])
            print("  Knots spacing (in Angstrom) for refraction fit: ",
                  refraction_dict['knots_spacing'])
        elif refraction_dict['method'] == 'polynomial':
            print("  Modelling performed  with polynomials")
            print("  Chebyshev polynomial order for differential refraction fit: ",
                  refraction_dict['fit_order'])
        else:
            raise ValueError("ERROR: fitting method for differential refraction not implemented")

        print("  Number of iterations: ",
              refraction_dict['fit_iters'])
        print()

        """ Now each observation is divided by the reference spectrum, after being doppler-shifted to the observer RF
            The result is then used to model the flux variation
        """
        approach = refraction_dict.get('approach', 'full_spectrum')

        if approach == 'full_spectrum':

            for obs in lists['observations']:
                print("  Division by reference spectrum and fit of the flux variation: ", obs)

                refraction[obs] = {}
                processed[obs] = {}

                if preparation['absolute_SRF']:
                    rv_shift = observational_pams[obs]['rv_shift_ORF2SRF']
                else:
                    rv_shift = observational_pams[obs]['rv_shift_ORF2SRF_mod']

                processed[obs]['ratio'] = preparation[obs]['rescaled'] / preparation['coadd']['rescaled']

                processed[obs]['ratio_err'] = processed[obs]['ratio'] * \
                    np.sqrt((preparation[obs]['rescaled_err']/preparation[obs]['rescaled'])**2
                            + (preparation['coadd']['rescaled_err']/preparation['coadd']['rescaled'])**2)

                refraction[obs]['fit_flag'] = (processed[obs]['ratio'] > 0.01)

                if refraction_dict['method'] == 'spline':
                    for n_iter in range(0, refraction_dict['fit_iters']):
                        wave = processed['coadd']['wave'][refraction[obs]['fit_flag']]

                        """ picking the number of knots """
                        nknots = ((np.amax(wave) - np.amin(wave)) / refraction_dict['knots_spacing'])
                        """ picking the indices of the knots"""
                        idx_knots = (np.arange(1, len(wave) - 1, (len(wave) - 2.) / nknots)).astype('int')
                        """ passing from indices to knots values """
                        processed[obs]['knots'] = wave[idx_knots]

                        refraction[obs]['coeff'] = \
                            sci_int.splrep(
                                processed['coadd']['wave'][refraction[obs]['fit_flag']],
                                processed[obs]['ratio'][refraction[obs]['fit_flag']],
                                task=-1,
                                k=refraction_dict['fit_order'],
                                t=processed[obs]['knots'])

                        refraction[obs]['fit_s1d'] = sci_int.splev(processed['coadd']['wave'], refraction[obs]['coeff'])
                        processed[obs]['residuals'] = processed[obs]['ratio'] - refraction[obs]['fit_s1d']

                        if n_iter < refraction_dict['fit_iters'] - 1:
                            std = np.std(processed[obs]['residuals'])
                            refraction[obs]['fit_flag'] = (refraction[obs]['fit_flag']) \
                                & (np.abs(processed[obs]['residuals']) <
                                   refraction_dict['fit_sigma'] * std)

                elif refraction_dict['method'] == 'polynomial':

                    refraction[obs]['fit_flag'][:50] = False
                    refraction[obs]['fit_flag'][-50:] = False

                    for n_iter in range(0, refraction_dict['fit_iters']):

                        refraction[obs]['coeff'] = np.polynomial.chebyshev.chebfit(
                            processed['coadd']['wave'][refraction[obs]['fit_flag']],
                            processed[obs]['ratio'][refraction[obs]['fit_flag']],
                            refraction_dict['fit_order'])

                        refraction[obs]['fit_s1d'] = \
                            np.polynomial.chebyshev.chebval(processed['coadd']['wave'], refraction[obs]['coeff'])

                        processed[obs]['residuals'] = processed[obs]['ratio'] - refraction[obs]['fit_s1d']

                        if n_iter < refraction_dict['fit_iters'] - 1:
                            std = np.std(processed[obs]['residuals'])
                            refraction[obs]['fit_flag'] = (refraction[obs]['fit_flag']) \
                                & (np.abs(processed[obs]['residuals']) <
                                   refraction_dict['fit_sigma'] * std)

                """ Going back to the observer RF and rebinning the polynomial fit into the observed orders """
                refraction[obs]['fit_e2ds'] = \
                    rebin_1d_to_2d(processed['coadd']['wave'],
                                   input_data['coadd']['step'],
                                   refraction[obs]['fit_s1d'],
                                   input_data[obs]['wave'],
                                   input_data[obs]['step'],
                                   rv_shift=-rv_shift,
                                   preserve_flux=False,
                                   )

                """ Zero or negative values are identified, flagged and substituted with another value """
                refraction[obs]['null'] = np.zeros([input_data[obs]['n_orders'], input_data[obs]['n_pixels']],
                                                   dtype=bool)
                for order in range(0, observational_pams['n_orders']):
                    refraction[obs]['fit_e2ds'][order, :], _, refraction[obs]['null'][order, :] = \
                        replace_values_errors_with_interpolation_1d(refraction[obs]['fit_e2ds'][order, :],
                                                                    refraction[obs]['fit_e2ds'][order, :],
                                                                    less_than=0.001)

                processed[obs]['flux_rebinned_stellarRF_corrected'] = preparation[obs]['flux_rebinned_stellarRF'] \
                    / refraction[obs]['fit_s1d']
                refraction[obs]['binned_residuals'] = \
                    rebin_1d_to_1d(processed['coadd']['wave'],
                                   processed['coadd']['step'],
                                   processed[obs]['residuals'],
                                   refraction['binned']['wave'],
                                   refraction['binned']['step'],
                                   preserve_flux=False)

        elif approach == 'individual_order':

            for obs in lists['observations']:

                print("  Division by reference spectrum and fit of the flux variation: ", obs)

                refraction[obs] = {}
                processed[obs] = {}

                if preparation['absolute_SRF']:
                    rv_shift = observational_pams[obs]['rv_shift_ORF2SRF']
                else:
                    rv_shift = observational_pams[obs]['rv_shift_ORF2SRF_mod']

                """ Going back to the observer RF and rebinning the spectrum into the observed orders """
                preserve_flux = input_data[obs].get('absolute_flux', True)
                # processed[obs]['master_flux'] = \
                processed[obs]['master_flux'] = \
                    rebin_1d_to_2d(preparation['coadd']['wave'],
                                   preparation['coadd']['step'],
                                   preparation['coadd']['rescaled'],
                                   input_data[obs]['wave'],
                                   input_data[obs]['step'],
                                   preserve_flux=preserve_flux,
                                   rv_shift=-rv_shift)

                # processed[obs]['master_ferr'] = \
                processed[obs]['master_ferr'] = \
                    rebin_1d_to_2d(preparation['coadd']['wave'],
                                   preparation['coadd']['step'],
                                   preparation['coadd']['rescaled_err'],
                                   input_data[obs]['wave'],
                                   input_data[obs]['step'],
                                   preserve_flux=preserve_flux,
                                   rv_shift=-rv_shift,
                                   is_error=True)

                """ Zero or negative values are identified, flagged and substituted with another value """
                processed[obs]['master_flux'], processed[obs]['master_ferr'], processed[obs]['master_null'] = \
                    replace_values_errors_with_interpolation_2d(processed[obs]['master_flux'],
                                                                processed[obs]['master_ferr'],
                                                                less_than=0.001)

                # processed[obs]['ratio'] = preparation[obs]['rescaled_blazed'] / master_flux

                processed[obs]['ratio'] = input_data[obs]['e2ds'] \
                    / preparation[obs]['rescaling'] \
                    / (processed[obs]['master_flux'] * calib_data['blaze'])

                processed[obs]['ratio_err'] = processed[obs]['ratio'] * \
                    np.sqrt(preparation[obs]['rescaling']/input_data[obs]['e2ds']
                            + (processed[obs]['master_ferr']/processed[obs]['master_flux'])**2)

                refraction[obs]['fit_e2ds'] = np.zeros([input_data[obs]['n_orders'], input_data[obs]['n_pixels']])
                processed[obs]['residuals'] = np.zeros([input_data[obs]['n_orders'], input_data[obs]['n_pixels']])
                refraction[obs]['fit_flag'] = np.zeros([input_data[obs]['n_orders'], input_data[obs]['n_pixels']],
                                                       dtype=bool)

                for order in range(0, input_data[obs]['n_orders']):

                    order_coeff_name = 'order_' + repr(order)
                    refraction[obs]['fit_flag'][order, :] = (processed[obs]['ratio'][order, :] > 0.1)

                    if refraction_dict['method'] == 'spline':

                        for n_iter in range(0, refraction_dict['fit_iters']):

                            wave = input_data[obs]['wave'][order, refraction[obs]['fit_flag'][order, :]]

                            """ picking the number of knots """
                            nknots = ((np.amax(wave) - np.amin(wave)) / refraction_dict['knots_spacing'])
                            """ picking the indices of the knots"""
                            idx_knots = (np.arange(1, len(wave) - 1, (len(wave) - 2.) / nknots)).astype('int')
                            """ passing from indices to knots values """
                            refraction[obs]['knots'] = wave[idx_knots]

                            refraction[obs][order_coeff_name] = \
                                sci_int.splrep(
                                    input_data[obs]['wave'][order, refraction[obs]['fit_flag'][order, :]],
                                    processed[obs]['ratio'][order, refraction[obs]['fit_flag'][order, :]],
                                    task=-1,
                                    k=refraction_dict['fit_order'],
                                    t=refraction[obs]['knots'])

                            refraction[obs]['fit_e2ds'][order, :] = \
                                sci_int.splev(input_data[obs]['wave'][order, :],
                                              refraction[obs][order_coeff_name])

                            processed[obs]['residuals'][order, :] = processed[obs]['ratio'][order, :] \
                                - refraction[obs]['fit_e2ds'][order, :]

                            if n_iter < refraction_dict['fit_iters'] - 1:
                                std = np.std(processed[obs]['residuals'][order, :])
                                refraction[obs]['fit_flag'][order, :] = (refraction[obs]['fit_flag'][order, :]) \
                                    & (np.abs(processed[obs]['residuals'][order, :]) <
                                       refraction_dict['fit_sigma'] * std)

                    elif refraction_dict['method'] == 'polynomial':
                        refraction[obs]['fit_flag'][order, :50] = False
                        refraction[obs]['fit_flag'][order, -50:] = False

                        for n_iter in range(0, refraction_dict['fit_iters']):

                            refraction[obs][order_coeff_name] = np.polynomial.chebyshev.chebfit(
                                input_data[obs]['wave'][order, refraction[obs]['fit_flag'][order, :]],
                                processed[obs]['ratio'][order, refraction[obs]['fit_flag'][order, :]],
                                refraction_dict['fit_order'])

                            refraction[obs]['fit_e2ds'][order, :] = \
                                np.polynomial.chebyshev.chebval(input_data[obs]['wave'][order, :],
                                                                refraction[obs][order_coeff_name])

                            processed[obs]['residuals'][order, :] = processed[obs]['ratio'][order, :] \
                                - refraction[obs]['fit_e2ds'][order, :]

                            if n_iter < refraction_dict['fit_iters'] - 1:
                                std = np.std(processed[obs]['residuals'][order, :])
                                refraction[obs]['fit_flag'][order, :] = (refraction[obs]['fit_flag'][order, :]) \
                                    & (np.abs(processed[obs]['residuals'][order, :]) <
                                       refraction_dict['fit_sigma'] * std)

                e2ds_corrected = input_data[obs]['e2ds'] / refraction[obs]['fit_e2ds']
                e2ds_corrected_err = input_data[obs]['e2ds_err'] / refraction[obs]['fit_e2ds']

                preserve_flux = input_data[obs].get('absolute_flux', True)
                processed[obs]['flux_rebinned_stellarRF_corrected'] = \
                    rebin_2d_to_1d(input_data[obs]['wave'],
                                   input_data[obs]['step'],
                                   e2ds_corrected,
                                   calib_data['blaze'],
                                   processed['coadd']['wave'],
                                   input_data['coadd']['step'],
                                   preserve_flux=preserve_flux,
                                   rv_shift=rv_shift)

                processed[obs]['err_flux_rebinned_stellarRF_corrected'] = \
                    rebin_2d_to_1d(input_data[obs]['wave'],
                                   input_data[obs]['step'],
                                   e2ds_corrected_err,
                                   calib_data['blaze'],
                                   processed['coadd']['wave'],
                                   input_data['coadd']['step'],
                                   preserve_flux=preserve_flux,
                                   rv_shift=rv_shift,
                                   is_error=True)

                processed[obs]['flux_rebinned_stellarRF_corrected'], \
                    processed[obs]['err_flux_rebinned_stellarRF_corrected'], _ = \
                    replace_values_errors_with_interpolation_1d(processed[obs]['flux_rebinned_stellarRF_corrected'],
                                                                processed[obs]['err_flux_rebinned_stellarRF_corrected'],
                                                                less_than=0.001)

                refraction[obs]['binned_residuals'] = \
                    rebin_2d_to_1d(input_data[obs]['wave'],
                                   input_data[obs]['step'],
                                   processed[obs]['residuals'],
                                   np.ones_like(processed[obs]['residuals']),
                                   refraction['binned']['wave'],
                                   refraction['binned']['step'],
                                   rv_shift=0.0000,
                                   preserve_flux=False)

        else:
            print("   Please choose either full_spectrum or individual_order as preferred approach")
            quit()

        if not config_in['settings'].get('full_output', False):
            for obs in lists['observations']:

                del processed[obs]['ratio_err']

                try:
                    del processed[obs]['err_flux_rebinned_stellarRF_corrected']
                    del processed[obs]['master_flux']
                    del processed[obs]['master_ferr']
                    del processed[obs]['master_null']
                except:
                    pass

        if append_name:
            save_to_cpickle('refraction_' + append_name + '_processed', processed, config_in['output'], night)
            save_to_cpickle('refraction_' + append_name, refraction, config_in['output'], night)
        else:
            save_to_cpickle('refraction_processed', processed, config_in['output'], night)
            save_to_cpickle('refraction', refraction, config_in['output'], night)


def plot_differential_refraction(config_in, night_input='', append_name=None):
    night_dict = from_config_get_nights(config_in)

    if night_input == '':
        night_list = night_dict
    else:
        night_list = np.atleast_1d(night_input)

    for night in night_list:

        refraction_dict = from_config_refraction(config_in, night)

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)

        """ Retrieving the observations"""
        input_data = retrieve_observations(config_in['output'], night, lists['observations'],
                                           use_refraction=False, use_telluric=False)
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        try:
            """ Retrieving the analysis"""
            if append_name:
                processed = load_from_cpickle('refraction_' + append_name + '_processed', config_in['output'], night)
                preparation = load_from_cpickle('refraction_' + append_name + '_preparation', config_in['output'],
                                                night)
                refraction = load_from_cpickle('refraction_' + append_name, config_in['output'], night)
            else:
                processed = load_from_cpickle('refraction_processed', config_in['output'], night)
                preparation = load_from_cpickle('refraction_preparation', config_in['output'], night)
                refraction = load_from_cpickle('refraction', config_in['output'], night)
        except:
            print("  Failed in retrieving processed data")
            return

        approach = refraction_dict.get('approach', 'full_spectrum')

        colors_properties, colors_plot, colors_scatter = make_color_array_matplotlib3(lists, observational_pams)

        offset = 0.10

        y_limits = [0.8, 1.2]

        flag_e2ds = {}
        flag_coadd = {}
        for i, obs in enumerate(lists['observations']):

            shrink_factor = 4
            if input_data[obs]['n_orders'] > shrink_factor:
                factor = (input_data[obs]['n_orders'] * input_data[obs]['n_pixels']) \
                    // (input_data[obs]['n_pixels'] * shrink_factor)
                flag_e2ds[obs] = (np.random.choice(a=([False] * (factor-1)) + [True],
                                                   size=(input_data[obs]['n_orders'], input_data[obs]['n_pixels'])))

                flag_coadd[obs] = \
                    np.random.choice(a=([False] * factor) + [True],  size=input_data['coadd']['size'])

            else:
                flag_e2ds[obs] = np.ones([input_data[obs]['n_orders'], input_data[obs]['n_pixels']], dtype=bool)
                flag_coadd[obs] = np.ones(input_data['coadd']['size'], dtype=bool)

        fig = plt.figure(figsize=(12, 6))
        gs = GridSpec(2, 2, width_ratios=[50, 1])

        ax1 = plt.subplot(gs[0, 0])
        ax2 = plt.subplot(gs[1, 0], sharex=ax1, sharey=ax1)
        cbax1 = plt.subplot(gs[:, 1])

        for i, obs in enumerate(lists['observations']):

            """  We slim down the plot  """

            if i == 0:
                ax1.scatter(preparation['coadd']['wave'][flag_coadd[obs]],
                            preparation[obs]['flux_rebinned_stellarRF'][flag_coadd[obs]] /
                            preparation[obs]['rescaling'],
                            c=colors_scatter['mBJD'][obs], s=2, alpha=0.2, label='observation (SRF)')
            else:
                ax1.scatter(preparation['coadd']['wave'][flag_coadd[obs]],
                            preparation[obs]['flux_rebinned_stellarRF'][flag_coadd[obs]] /
                            preparation[obs]['rescaling'],
                            c=colors_scatter['mBJD'][obs], s=2, alpha=0.2)

            ax2.scatter(processed['coadd']['wave'][flag_coadd[obs]],
                        processed[obs]['flux_rebinned_stellarRF_corrected'][flag_coadd[obs]] /
                        preparation[obs]['rescaling'],
                        c=colors_scatter['mBJD'][obs], s=3, alpha=0.2)

        ax1.plot(preparation['coadd']['wave'], preparation['coadd']['rescaled'], c='k', lw=1, alpha=0.5,
                 label='reference spectrum')
        ax2.plot(preparation['coadd']['wave'], preparation['coadd']['rescaled'], c='k', lw=1, alpha=0.5)

        ax1.set_xlim(processed['coadd']['wave'][0], processed['coadd']['wave'][-1])
        ax1.set_ylim(y_limits)
        ax2.set_ylim(y_limits)
        ax1.legend(loc=1)
        ax1.set_title('Night: {0:s} \n Input spectra'.format(night))
        ax2.set_title('After differential refraction correction')
        ax2.set_xlabel('$\lambda$ [$\AA$]')

        sm = plt.cm.ScalarMappable(cmap=colors_properties['cmap'], norm=colors_properties['norm']['mBJD'])
        sm.set_array([])  # You have to set a dummy-array for this to work...
        cbar = plt.colorbar(sm, cax=cbax1)
        cbar.set_label('BJD - 2450000.0')
        fig.subplots_adjust(wspace=0.05, hspace=0.4)
        plt.show()

        """
        PLOT
        """

        fig = plt.figure(figsize=(12, 6))
        gs = GridSpec(1, 2, width_ratios=[50, 1])
        ax = plt.subplot(gs[0, 0])
        cbax1 = plt.subplot(gs[:, 1])

        for i, obs in enumerate(lists['observations']):

            if i == 0:
                offset = np.std(processed[obs]['ratio'][refraction[obs]['fit_flag']].flatten()) * 6
                average = np.average(processed[obs]['ratio'][refraction[obs]['fit_flag']].flatten())
                y_limits = [average - offset, average + offset]

            if approach == 'full_spectrum':
                flag = flag_coadd[obs] & refraction[obs]['fit_flag']
                wave = refraction['wave']

            elif approach == 'individual_order':
                flag = flag_e2ds[obs] & refraction[obs]['fit_flag']
                wave = input_data[obs]['wave']

            ax.scatter(wave[flag],
                       processed[obs]['ratio'][flag] + offset * i,
                       c=colors_scatter['mBJD'][obs], s=1, alpha=0.50, zorder=2)

            ax.scatter(wave[~refraction[obs]['fit_flag']],
                       processed[obs]['ratio'][~refraction[obs]['fit_flag']] + offset * i,
                       c='k', s=2, alpha=0.1, zorder=1)

            for order in range(0, input_data[obs]['n_orders']):

                ax.plot(input_data[obs]['wave'][order, :],
                        refraction[obs]['fit_e2ds'][order, :] + offset * i,
                        c='k', lw=1, alpha=0.5, zorder=5)

        y_limits_offset = [min(y_limits[0] + offset * i, y_limits[0]),
                           max(y_limits[1] + offset * i, y_limits[1])]

        ax.set_ylim(y_limits_offset)
        ax.set_xlabel('$\lambda$ [$\AA$]')
        # ax.legend(loc=3)
        ax.set_title('Night: {0:s} \n Differential refraction correction - Fit of the ratio obs/master'.format(night))

        sm = plt.cm.ScalarMappable(cmap=colors_properties['cmap'], norm=colors_properties['norm']['mBJD'])
        sm.set_array([])  # You have to set a dummy-array for this to work...
        cbar = plt.colorbar(sm, cax=cbax1)
        cbar.set_label('BJD - 2450000.0')
        fig.subplots_adjust(wspace=0.05, hspace=0.4)
        plt.show()

        """
        PLOT: residuals of the fit
        """

        fig = plt.figure(figsize=(12, 6))
        gs = GridSpec(1, 2, width_ratios=[50, 1])
        ax = plt.subplot(gs[0, 0])
        cbax1 = plt.subplot(gs[:, 1])

        approach = refraction_dict.get('approach', 'full_spectrum')

        for i, obs in enumerate(lists['observations']):

            if i == 0:
                median = np.median(processed[obs]['residuals'][refraction[obs]['fit_flag']].flatten())
                offset = np.std(processed[obs]['residuals'][refraction[obs]['fit_flag']].flatten()) * 6
                y_limits = [median - offset, median + offset]

            if approach == 'full_spectrum':
                flag = flag_coadd[obs] & refraction[obs]['fit_flag']
                wave = refraction['wave']

            elif approach == 'individual_order':
                flag = flag_e2ds[obs] & refraction[obs]['fit_flag']
                wave = input_data[obs]['wave']

            ax.scatter(wave[flag],
                       processed[obs]['residuals'][flag] + offset * i,
                       c=colors_scatter['mBJD'][obs], s=1, alpha=0.50, zorder=2)

            ax.scatter(wave[~refraction[obs]['fit_flag']],
                       processed[obs]['residuals'][~refraction[obs]['fit_flag']] + offset * i,
                       c='k', s=2, alpha=0.1, zorder=1)

            ax.axhline(offset * i, c='k', zorder=3)

        y_limits_offset = [min(y_limits[0] + offset * i, y_limits[0]),
                           max(y_limits[1] + offset * i, y_limits[1])]

        ax.set_ylim(y_limits_offset)
        ax.set_xlabel('$\lambda$ [$\AA$]')
        # ax.legend(loc=3)
        ax.set_title(
            'Night: {0:s} \n Differential refraction correction - Residuals of the fit on ratio obs/master'.format(
                night))

        sm = plt.cm.ScalarMappable(cmap=colors_properties['cmap'], norm=colors_properties['norm']['mBJD'])
        sm.set_array([])  # You have to set a dummy-array for this to work...
        cbar = plt.colorbar(sm, cax=cbax1)
        cbar.set_label('BJD - 2450000.0')
        fig.subplots_adjust(wspace=0.05, hspace=0.4)
        plt.show()

        continue

        """
        PLOT: corrected e2ds
        """

        fig = plt.figure(figsize=(12, 6))
        gs = GridSpec(1, 2, width_ratios=[50, 1])
        ax = plt.subplot(gs[0, 0])
        cbax1 = plt.subplot(gs[:, 1])

        for i, obs in enumerate(lists['observations']):

            e2ds_corrected = input_data[obs]['e2ds'] / refraction[obs]['fit_e2ds']

            ax.scatter(input_data[obs]['wave'],
                       e2ds_corrected / preparation[obs]['rescaling'],
                       c=colors_scatter['mBJD'][obs], s=2, alpha=0.20)

            # for order in range(0, np.size(input_data[obs]['wave'][:, 0])):
            #
            #    ax.plot(input_data[obs]['wave'][order, :],
            #            refraction[obs]['fit_e2ds'][order, :],
            #            c=color_array, lw=1)

        ax.set_xlabel('$\lambda$ [$\AA$]')
        ax.set_title('Night: {0:s} \n Rescaled e2ds spectra after differential refraction correction'.format(night))

        sm = plt.cm.ScalarMappable(cmap=colors_properties['cmap'], norm=colors_properties['norm']['mBJD'])
        sm.set_array([])  # You have to set a dummy-array for this to work...
        cbar = plt.colorbar(sm, cax=cbax1)
        cbar.set_label('BJD - 2450000.0')
        fig.subplots_adjust(wspace=0.05, hspace=0.4)
        plt.show()



from setuptools import setup

# Inspired from here:
# https://hynek.me/articles/sharing-your-labor-of-love-pypi-quick-and-dirty/
# https://realpython.com/pypi-publish-python-package/#prepare-your-package-for-publication

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="SLOPpy-package",
    version='1.5.1',
    author="Daniela Sicilia, Luca Malavolta, et al.",
	author_email = 'daniela.sicilia@inaf.it, luca.malavolta@unipd.it',
	url = 'https://github.com/LucaMalavolta/SLOPpy',
	packages =['SLOPpy', 'SLOPpy.subroutines', 'SLOPpy.instruments'],
	license = 'MIT License',
	description ='SLOPpy: Spectral Lines Of Planets with python',
    long_description=long_description,
    long_description_content_type='text/markdown',
	classifiers = [
		'Development Status :: 5 - Production/Stable',
		'Intended Audience :: Science/Research',
		'Topic :: Scientific/Engineering',
        'Operating System :: OS Independent',
		'Programming Language :: Python :: 3'
		],
    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            'SLOPpy_run=SLOPpy.sloppy_run:sloppy_run',
        ]
    },
    zip_safe=False,
    install_requires=[
        'numpy==1.26.4',
        'numba>=0.55.1',
        'scipy>=1.8.1',
        'matplotlib>=3.5.2',
        'astropy>=5.1',
        'astroquery>=0.4',
        'pyerfa>=2.0',
        'argparse>=1.4',
        'oyaml>=1.0',
        'emcee>=3.1.2',
        'pyyaml',
        'h5py>=3.7.0',
        'tqdm>=4.60',
        'pygtc>=0.4.1',
        'jaxlib==0.4.31',
        'jax==0.4.31',
        'tinygp>=0.3.0',
        'PyAstronomy>=0.18',
        'ordered-set',
        'scikit-learn'
    ],
    setup_requires=['setuptools']
)



from __future__ import print_function, division
import numpy as np
import SLOPpy.subroutines.kepler_exo as kp

__all__ = ["StarParameters", "PlanetParameters"]


G_grav = 6.67428e-11 # Gravitational Constants in SI system [m^3/kg/s^2]
M_sun = 1.9884e30 # Value from TRADES

class StarParameters:
    """
    This class is just a stub with basic properties, to be expanded in th efuture
    E.g.: stellar mass, radius, activity level, etc
    """
    def __init__(self, mass=None, radius=None):
        self.kind = 'star'
        self.mass = mass
        self.radius = radius


class PlanetParameters:
    """
    This class is just a stub with basic properties, to be expanded in the future
    E.g.: give a planetary mass and convert it to expected K, and vice versa
    """
    def __init__(self):
        #array of shape (2), error should be stored here
        self.kind = 'planet'
        self.star_mass = None
        self.star_radius = None

        self.reference_epoch = None
        self.reference_transit = None

        self.period = None
        self.phase = None
        self.eccentricity = None
        self.omega = None

        self.RV_semiamplitude = None
        self.RVplanet_semiamplitude = None

        self.radius = None  # stellar radius in stellar unit
        self.area = None
        self.mass = None    # planet mass (useless for now)
        self.inclination = None #orbital inclination
        self.impact_parameter = None
        self.semimajor_axis = None

        self.gamma = None # RV zero point (including systematic RV of the star)

    def put_reference_epoch(self, Tref):
        self.reference_epoch = Tref

    def put_gamma(self, gamma):
        self.gamma = gamma

    def put_star(self, star):
        self.star_mass = star.mass
        self.star_radius = star.radius

    def put_RVparameters(self, P, K, Tc, e, o, degree=False):

        self.period = P
        self.RV_semiamplitude = K
        self.reference_transit = Tc
        self.eccentricity = e
        # Angles must be in radians
        if degree:
            self.omega = o/180. * np.pi
        else:
            self.omega = o

        self.phase = kp.kepler_Tc2phase_Tref(self.period,
                                             self.reference_transit,
                                             self.eccentricity,
                                             self.omega)

    def put_Transitparameters(self, Rp, i, b, a, degree=True):


        # degree=True by default because inclination is always expressed in degrees
        self.radius = Rp
        self.inclination = i
        self.impact_parameter = b
        self.semimajor_axis = a

    def put_RVplanet(self, Kplanet):
        self.RVplanet_semiamplitude = Kplanet

    def get_RV_kms(self, bjd):
        return self.get_RV_ms(bjd)/1000.000

    def get_RV_ms(self, bjd):
        return kp.kepler_RV_T0P(bjd-self.reference_epoch, self.phase, self.period,
                                self.RV_semiamplitude, self.eccentricity, self.omega)

    def get_RVplanet_kms(self, bjd):
        return self.get_RVplanet_ms(bjd)/1000.000

    def get_RVplanet_ms(self, bjd):
        return (-1.)*kp.kepler_RV_T0P(bjd-self.reference_epoch, self.phase, self.period,
                                self.RVplanet_semiamplitude, self.eccentricity, self.omega)
